@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <kgame/kgameio.h>
 
 class Player;
+class Unit;
 
 /**
  * @author Andreas Beckermann <b_mann@gmx.de>
@@ -29,21 +30,22 @@ class Player;
 class BosonComputerIO : public KGameComputerIO
 {
 	Q_OBJECT
-	enum RTTI {
-		IdBosonComputerIO = 500
-	};
+
 public:
 	BosonComputerIO();
 	BosonComputerIO(KPlayer*);
 	~BosonComputerIO();
 
-	virtual int rtti() const { return IdBosonComputerIO; }
+	virtual int rtti() const { return KGameIO::ComputerIO; }
 
 protected:
 	virtual void reaction();
 	Player* boPlayer() const { return (Player*)player(); }
+	Unit* findTarget();
 
 private:
+	int mUnit;
+	Unit* mTarget;
 };
 
 #endif
