@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,15 +29,12 @@
 #include <qdatetime.h>
 #include <qdatastream.h>
 #include <qdom.h>
+#include <qimage.h>
 
 #include <kapplication.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
 
-#define TAG_FIELD "boeditor_magic_0_6"
-#define TAG_FIELD_LEN 18 // len of above
-#define TAG_CELL (0xde)
-
 class BosonMap::BosonMapPrivate
 {
 public:
@@ -55,9 +52,8 @@ BosonMap::BosonMap(QObject* parent) : QObject(parent)
 BosonMap::~BosonMap()
 {
  delete mTiles;
- if (mCells) {
-	delete[] mCells;
- }
+ delete[] mCells;
+ delete[] mHeightMap;
  delete d;
 }
 
@@ -65,117 +61,78 @@ void BosonMap::init()
 {
  d = new BosonMapPrivate;
  mCells = 0;
+ mHeightMap = 0;
  mTiles = 0;
  setModified(false);
 }
 
-bool BosonMap::loadMap(const QByteArray& buffer, bool binary)
-{
- if (binary) {
-	QDataStream stream(buffer, IO_ReadOnly);
-	if (!verifyMap (stream)) { // we already did this so this cannot fail
-		boError() << k_funcinfo << "Invalid map file" << endl;
-		return false;
-	}
-	if (!loadMapGeo(stream)) {
-		boError() << "Error loading map geo" << endl;
-		return false;
-	}
-	if (!loadCells(stream)) {
-		boError() << "Error loading map cells" << endl;
-		return false;
-	}
-	return true;
- }
-
- // load XML file
- QDomDocument doc("BosonMap");
- QString errorMsg;
- int lineNo;
- int columnNo;
- if (!doc.setContent(buffer, false, &errorMsg, &lineNo, &columnNo)) {
-	boError() << "Parse error in line " << lineNo << ",column " << columnNo
-			<< " error message: " << errorMsg << endl;
-	return false;
- }
- QDomElement root = doc.documentElement();
- return loadMap(root);
-}
-
 bool BosonMap::loadMap(QDomElement& root)
 {
  QDomNodeList list;
  list = root.elementsByTagName("MapGeo");
  if (list.count() != 1) {
-	boError() << "XML error: cannot have tag MapGeo " 
+	boError() << k_funcinfo << "XML error: cannot have tag MapGeo "
 			<< list.count() << " times" << endl;
 	return false;
  }
  QDomElement geo = list.item(0).toElement();
  if (geo.isNull()) {
-	boError() << "XML error: geo is not an QDomElement" << endl;
+	boError() << k_funcinfo << "XML error: geo is not an QDomElement" << endl;
 	return false;
  }
  if (!loadMapGeo(geo)) {
-	boError() << "XML error: failed loading map geo" << endl;
+	boError() << k_funcinfo << "XML error: failed loading map geo" << endl;
 	return false;
  }
 
  list = root.elementsByTagName("MapCells");
  if (list.count() != 1) {
-	boError() << "XML error: cannot have tag Map Geo " 
+	boError() << k_funcinfo << "XML error: cannot have tag Map Geo " 
 			<< list.count() << " times" << endl;
 	return false;
  }
  QDomElement cells = list.item(0).toElement();
  if (geo.isNull()) {
-	boError() << "XML error: cells is not an QDomElement" << endl;
+	boError() << k_funcinfo << "XML error: cells is not an QDomElement" << endl;
 	return false;
  }
  if (!loadCells(cells)) {
-	boError() << "XML error: failed loading map geo" << endl;
+	boError() << k_funcinfo << "XML error: failed loading map geo" << endl;
 	return false;
  }
 
  return true;
 }
 
-bool BosonMap::verifyMap(QDataStream& stream)
+bool BosonMap::loadMapFromFile(QDataStream& stream)
 {
- char magic[ TAG_FIELD_LEN+4 ];  // paranoic spaces
- int i;
-
- // magic 
- // Qt marshalling for a string is 4-byte-len + data
- stream >> i;
- if (TAG_FIELD_LEN + 1 != i) {
-//	boError() << k_funcinfo << "Magic doesn't match(len), check file name" << endl;// not an error - probably an XML file
+ if (!loadMapGeo(stream)) {
+	boError() << k_funcinfo << "Could not load map geo" << endl;
 	return false;
  }
-
- for (i = 0; i < TAG_FIELD_LEN + 1; i++) {
-	Q_INT8  b;
-	stream >> b;
-	magic[i] = b;
- }
-
- if (strncmp(magic, TAG_FIELD, TAG_FIELD_LEN) ) {
-//	boError() << k_funcinfo << "Magic doesn't match(string), check file name" << endl;
+ if (!loadCells(stream)) {
+	boError() << k_funcinfo << "Could not load map cells" << endl;
 	return false;
  }
  return true;
 }
 
-void BosonMap::saveValidityHeader(QDataStream& stream)
-{ // TODO: use QString for this
- // magic 
- // Qt marshalling for a string is 4-byte-len + data
 
- stream << (Q_INT32)(TAG_FIELD_LEN - 1);
-
- for (int i = 0; i <= TAG_FIELD_LEN; i++) {
-	stream << (Q_INT8)TAG_FIELD[i];
+bool BosonMap::loadMap(QDataStream& stream)
+{
+ if (!loadMapGeo(stream)) {
+	boError() << k_funcinfo << "Could not load map geo" << endl;
+	return false;
  }
+ if (!loadCells(stream)) {
+	boError() << k_funcinfo << "Could not load map cells" << endl;
+	return false;
+ }
+ if (!loadHeightMap(stream)) {
+	boError() << k_funcinfo << "Could not load height map" << endl;
+	return false;
+ }
+ return true;
 }
 
 bool BosonMap::loadMapGeo(QDataStream& stream)
@@ -185,7 +142,7 @@ bool BosonMap::loadMapGeo(QDataStream& stream)
 
  stream >> mapWidth;
  stream >> mapHeight;
- 
+
  // check 'realityness'
  if (mapWidth < 10) {
 	boError() << k_funcinfo << "broken map file!" << endl;
@@ -216,7 +173,12 @@ bool BosonMap::loadMapGeo(QDataStream& stream)
 //	boDebug() << "cells created before!! try to delete..." << endl;
 	delete[] mCells;
  }
+ if (mHeightMap) {
+	delete[] mHeightMap;
+ }
+
  mCells = new Cell[width() * height()];
+ mHeightMap = new float[(width() + 1) * (height() + 1)];
  for (unsigned int x = 0; x < width(); x++) {
 	for (unsigned y = 0; y < height(); y++) {
 		Cell* c = cell(x, y);
@@ -227,7 +189,7 @@ bool BosonMap::loadMapGeo(QDataStream& stream)
 		c->setPosition(x, y);
 	}
  }
- return true;
+ return loadHeightMapImage(QByteArray());
 }
 
 bool BosonMap::loadCells(QDataStream& stream)
@@ -250,48 +212,31 @@ bool BosonMap::loadCells(QDataStream& stream)
  return true;
 }
 
-
-bool BosonMap::saveMap(QDomElement& root)
+bool BosonMap::loadHeightMap(QDataStream& stream)
 {
- QDomDocument doc = root.ownerDocument();
- QDomElement node = doc.createElement("BosonMap");
- root.appendChild(node);
-
- QDomElement geo = doc.createElement("MapGeo");
- node.appendChild(geo);
-
- if (!saveMapGeo(geo)) {
-	boError() << k_funcinfo << "Could not save map geo" << endl;
+ if (!mHeightMap) {
+	boError() << k_funcinfo << "NULL heightmap" << endl;
 	return false;
  }
-
- QDomElement cells = doc.createElement("MapCells");
- node.appendChild(cells);
-
- if (!saveCells(cells)) {
-	boError() << k_funcinfo << "Could not save cells" << endl;
-	return false;
+ boDebug() << k_funcinfo << "loading height map from network stream" << endl;
+ for (unsigned int x = 0; x < width() + 1; x++) {
+	for (unsigned int y = 0; y < height() + 1; y++) {
+		stream >> mHeightMap[y * (width() + 1) + x];
+	}
  }
 
-return true;
-}
-
-bool BosonMap::saveMapGeo(QDomElement& node)
-{
-// TODO: maybe check for validity
- node.setAttribute("Width", width());
- node.setAttribute("Height", height());
  return true;
 }
 
+
 bool BosonMap::loadMapGeo(QDomElement& node)
 {
  if (!node.hasAttribute("Width")) {
-	boError() << "Map width is mandatory!" << endl;
+	boError() << k_funcinfo << "Map width is mandatory!" << endl;
 	return false;
  }
  if (!node.hasAttribute("Height")) {
-	boError() << "Map height is mandatory!" << endl;
+	boError() << k_funcinfo << "Map height is mandatory!" << endl;
 	return false;
  }
  Q_INT32 width = node.attribute("Width").toInt();
@@ -311,14 +256,14 @@ bool BosonMap::loadCells(QDomElement& node)
 {
  QDomNodeList list = node.elementsByTagName("Cell");
  if (list.count() < width() * height()) {
-	boError() << "XML error: not enough cells" << endl;
+	boError() << k_funcinfo << "XML error: not enough cells" << endl;
 	return false;
  }
  if (list.count() != width() * height()) {
-	boWarning() << "Cell count doesn't match width * height" 
+	boWarning() << k_funcinfo << "Cell count doesn't match width * height"
 			<< endl;
  }
- 
+
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  int* groundType = new int[width() * height()];
@@ -326,24 +271,24 @@ bool BosonMap::loadCells(QDomElement& node)
  for (unsigned int i = 0; i < list.count(); i++) {
 	QDomElement cell = list.item(i).toElement();
 	if (cell.isNull()) {
-		boError() << "XML error: cell is not an QDomElement" << endl;
+		boError() << k_funcinfo << "XML error: cell is not an QDomElement" << endl;
 	} else {
 		int x;
 		int y;
 		int g;
 		unsigned char v;
 		if (!loadCell(cell, x, y, g, v)) {
-			boError() << "XML error: could not load cell" << endl;
+			boError() << k_funcinfo << "XML error: could not load cell" << endl;
 			continue;
 		}
-		groundType[x + y * width()] = g;
-		version[x + y * width()] = v;
+		groundType[cellArrayPos(x, y)] = g;
+		version[cellArrayPos(x, y)] = v;
 	}
  }
  for (unsigned int i = 0; i < width(); i++) {
 	for (unsigned int j = 0; j < height(); j++) {
-		saveCell(stream, groundType[i + j * width()],
-				version[i + j * width()]);
+		saveCell(stream, groundType[cellArrayPos(i, j)],
+				version[cellArrayPos(i, j)]);
 	}
  }
 
@@ -354,22 +299,85 @@ bool BosonMap::loadCells(QDomElement& node)
  return loadCells(readStream);
 }
 
+bool BosonMap::loadHeightMapImage(const QByteArray& heightMap)
+{
+// boDebug() << k_funcinfo << endl;
+ if (heightMap.size() == 0) {
+	// initialize the height map with 0.0
+//	boDebug() << k_funcinfo << "loading dummy height map" << endl;
+	for (unsigned int x = 0; x < width() + 1; x++) {
+		for (unsigned int y = 0; y < height() + 1; y++) {
+			mHeightMap[y * (width() + 1) + x] = 0.0;
+		}
+	}
+	return true;
+ }
+// boDebug() << k_funcinfo << "loading real height map" << endl;
+ QImage map(heightMap);
+ if (!map.isGrayscale()) {
+	// we load a valid height map (i.e. a dummy height map) but still return
+	// false for error checking.
+	boError() << k_funcinfo << "not a grayscale image" << endl;
+	loadHeightMapImage(QByteArray());
+	return false;
+ }
+ if ((unsigned int)map.height() != height() + 1) {
+	boError() << k_funcinfo << "invalid height of heightmap: " <<
+			map.height() << " must be: " << height() + 1 << endl;
+	loadHeightMapImage(QByteArray());
+	return false;
+ }
+ if ((unsigned int)map.width() != width() + 1) {
+	boError() << k_funcinfo << "invalid widthof heightmap: "
+			<< map.width() << " must be: " << width() + 1 << endl;
+	loadHeightMapImage(QByteArray());
+	return false;
+ }
+ int increment = 1;
+ if (map.bytesPerLine() > map.width() + 1) {
+	// QT doesn't save images as grayscale. it returns bytesPerLine() ==
+	// width() (i.e. 8bits per pixel) for all *actual* grayscale images
+	// and bytesPerLine() == width() * 4 for grayscale images that Qt
+	// has created (i.e. RGB/RGBA images). we only use grayscale here, that
+	// means red=blue=green component. so we can just pick the first and
+	// skip all (including alpha) other values.
+	increment = 4;
+ }
+ for (unsigned int y = 0; y < width() + 1; y++) {
+	// AB: warning: from Qt docs: "If you are accessing 16-bpp image data,
+	// you must handle endianness yourself."
+	// do we have to care about this? (since we are using 16bpp)
+	// AB: we use 32bpp (qt doesnt support 16bpp on X11)
+	// AB: hmm accordint to "file" we use 6bpp only... ok thats easier
+	// then :)
+	int imageX = 0;
+	unsigned char* line = map.scanLine(y);
+	for (unsigned int x = 0; x < height() + 1; x++, imageX += increment) {
+		mHeightMap[y * (width() + 1) + x] = pixelToHeight(line[imageX]);
+	}
+ }
+
+ // No need to recalculate cell values here since actual values will be loaded
+ //  from network stream later... right?
+ return true;
+}
+
 bool BosonMap::loadCell(QDomElement& node, int& x, int& y, int& groundType, unsigned char& version)
 {
  if (!node.hasAttribute("x")) {
-	boError() << "XML: attribute x is mandatory!" << endl;
+	boError() << k_funcinfo << "XML: attribute x is mandatory!" << endl;
 	return false;
  }
  if (!node.hasAttribute("y")) {
-	boError() << "XML: attribute y is mandatory!" << endl;
+	boError() << k_funcinfo << "XML: attribute y is mandatory!" << endl;
 	return false;
  }
  if (!node.hasAttribute("GroundType")) {
-	boError() << "XML: attribute GroundType is mandatory!" << endl;
+	boError() << k_funcinfo << "XML: attribute GroundType is mandatory!" << endl;
 	return false;
  }
  if (!node.hasAttribute("Version")) {
-	boError() << "XML: attribute Version is mandatory!" << endl;
+	boError() << k_funcinfo << "XML: attribute Version is mandatory!" << endl;
 	return false;
  }
  x = node.attribute("x").toInt();
@@ -388,43 +396,40 @@ bool BosonMap::loadCell(QDomElement& node, int& x, int& y, int& groundType, unsi
  return true;
 }
 
-bool BosonMap::saveCells(QDomElement& node)
+bool BosonMap::saveMapToFile(QDataStream& stream)
 {
- if (!mCells) {
-	boError() << k_funcinfo << "NULL cells" << endl;
+ if (!saveMapGeo(stream)) {
+	boError() << k_funcinfo << "Could not save map geo" << endl;
 	return false;
  }
- QDomDocument doc = node.ownerDocument();
- for (unsigned int i = 0; i < width(); i++) {
-	for (unsigned int j = 0; j < height(); j++) {
-		QDomElement c = doc.createElement("Cell");
-		node.appendChild(c);
-		saveCell(c, i, j, cell(i, j));
-	}
+ if (!saveCells(stream)) {
+	boError() << k_funcinfo << "Could not save map cells" << endl;
+	return false;
  }
  return true;
 }
 
-bool BosonMap::saveCell(QDomElement& node, int x, int y, Cell* cell)
+bool BosonMap::saveMap(QDataStream& stream)
 {
- if (!cell) {
-	boError() << k_funcinfo << "NULL cell" << endl;
+ if (!saveMapGeo(stream)) {
+	boError() << k_funcinfo << "Could not save map geo" << endl;
+	return false;
+ }
+ if (!saveCells(stream)) {
+	boError() << k_funcinfo << "Could not save map cells" << endl;
+	return false;
+ }
+ if (!saveHeightMap(stream)) {
+	boError() << k_funcinfo << "Could not save height map" << endl;
 	return false;
  }
- QDomDocument doc = node.ownerDocument();
- node.setAttribute("x", x);
- node.setAttribute("y", y);
-
-// or should these be separate elements? I doubt this.
- node.setAttribute("GroundType", cell->groundType());
- node.setAttribute("Version", cell->version());
  return true;
 }
 
 bool BosonMap::saveMapGeo(QDataStream& stream)
 {
  if (!isValid()) {
-	boError() << "Map geo is not valid" << endl;
+	boError() << k_funcinfo << "Map geo is not valid" << endl;
 	return false;
  }
 // boDebug() << k_funcinfo << endl;
@@ -452,28 +457,102 @@ bool BosonMap::saveCells(QDataStream& stream)
  return true;
 }
 
+bool BosonMap::saveHeightMap(QDataStream& stream)
+{
+ if (!mHeightMap) {
+	stream << QImage();
+	return true;
+ }
+ if (!mHeightMap) {
+	boDebug() << k_funcinfo << "saving dummy heightmap to network stream" << endl;
+	for (unsigned int x = 0; x < width() + 1; x++) {
+		for (unsigned int y = 0; y < height() + 1; y++) {
+			stream << (float)0.0;
+		}
+	}
+	return true;
+ }
+ boDebug() << k_funcinfo << "saving real heightmap to network stream" << endl;
+ for (unsigned int x = 0; x < width() + 1; x++) {
+	for (unsigned int y = 0; y < height() + 1; y++) {
+		stream << (float)mHeightMap[y * (width() + 1) + x];
+	}
+ }
+ return true;
+}
+
+QByteArray BosonMap::saveHeightMapImage()
+{
+ // this function is sloooow !
+ if (!width() || !height()) {
+	boError() << k_funcinfo << "Cannot save empty map" << endl;
+	return QByteArray();
+ }
+ QImage image;
+ if (!image.create(width() + 1, height() + 1, 32, 0)) { // AB: 16bpp isnt available for X11 (only for qt embedded)
+	boError() << k_funcinfo << "Unable to create height map!" << endl;
+	return QByteArray();
+ }
+ boDebug() << k_funcinfo << "heightmap: " << image.width() << "x" << image.height() << endl;
+ if (!mHeightMap) {
+	boDebug() << k_funcinfo << "dummy height map..." << endl;
+	int l = heightToPixel(0.0f);
+	image.fill(l);
+ } else {
+	boDebug() << k_funcinfo << "real height map" << endl;
+	// AB: this *might* be correct, but i am not sure about this. (02/11/22)
+	for (int y = 0; y < image.height(); y++) {
+		uint* p = (uint*)image.scanLine(y);
+		for (int x = 0; x < image.width(); x++) {
+			float value = mHeightMap[y * (width() + 1) + x];
+			int v = heightToPixel(value);
+			*p = qRgb(v, v, v);
+			p++;
+		}
+	}
+ }
+
+ if (!image.isGrayscale()) {
+	boError() << k_funcinfo << "not a grayscale image!!" << endl;
+	return QByteArray();
+ }
+
+ QByteArray array;
+ QDataStream stream(array, IO_WriteOnly);
+ QImageIO io;
+ io.setIODevice(stream.device());
+ io.setFormat("PNG");
+ io.setImage(image);
+ io.write();
+ return array;
+}
+
 bool BosonMap::isValid() const
 {
  if (width() < 10) {
-	boError() << "width < 10" << endl;
+	boError() << k_funcinfo << "width < 10" << endl;
 	return false;
  }
  if (width() > MAX_MAP_WIDTH) {
-	boError() << "mapWidth > " << MAX_MAP_WIDTH << endl;
+	boError() << k_funcinfo << "mapWidth > " << MAX_MAP_WIDTH << endl;
 	return false;
  }
  if (height() < 10) {
-	boError() << "height < 10" << endl;
+	boError() << k_funcinfo << "height < 10" << endl;
 	return false;
  }
  if (height() > MAX_MAP_HEIGHT) {
-	boError() << "mapHeight > " << MAX_MAP_HEIGHT << endl;
+	boError() << k_funcinfo << "mapHeight > " << MAX_MAP_HEIGHT << endl;
 	return false;
  }
  if (!mCells) {
 	boError() << k_funcinfo << "NULL cells" << endl;
 	return false;
  }
+ if (!mHeightMap) {
+	boError() << k_funcinfo << "NULL heightmap" << endl;
+	return false;
+ }
 
  //TODO: check cells!
 
@@ -484,19 +563,12 @@ bool BosonMap::loadCell(QDataStream& stream, int& groundType, unsigned char& b)
 {
  Q_INT32 g;
  Q_INT8 version;
- stream >> g; // not groundType - first TAG_CELL
- if (g != TAG_CELL) {
-	boError() << k_funcinfo << "broken map file!" << endl;
-	boError() << "missing TAG_CELL!" << endl;
-	return false;
- }
 
- stream >> g; // this is the groundType now.
- if(!Cell::isValidGround(g)) { 
+ stream >> g;
+ if (!Cell::isValidGround(g)) { 
 	return false; 
  }
-// if (g < 0 || ) { return false; }
- 
+
  stream >> version;
  if (version > 4) {
 	boError() << k_funcinfo << "broken map file!" << endl;
@@ -515,7 +587,6 @@ void BosonMap::saveCell(QDataStream& stream, int groundType, unsigned char versi
 	boWarning() << k_funcinfo << "Invalid version " << version << endl;
 	version = 0;
  }
- stream << (Q_INT32)TAG_CELL;
  stream << (Q_INT32)groundType;
  stream << (Q_INT8)version;
 }
@@ -526,13 +597,47 @@ Cell* BosonMap::cell(int x, int y) const
 	boError() << k_funcinfo << "Cells not yet created" << endl;
 	return 0;
  }
- if (x < 0 || x >= (int)width()) {
+ if (!isValidCell(x, y)) {
 	return 0;
  }
- if (y < 0 || y >= (int)height()) {
-	return 0;
+ return mCells + cellArrayPos(x, y);
+}
+
+float BosonMap::heightAtCorner(int x, int y) const
+{
+ if (!mHeightMap) {
+	boError() << k_funcinfo << "NULL height map" << endl;
+	return 1.0f;
  }
- return &mCells[ x + y * width() ];
+ // note: isValidCell(x,y) might return false, even though the coordinates are
+ // valid.
+ // if x=width() and y=height() we will return the lower right corner of the
+ // cell at x=width()-1 and y=height()-1.
+ // This value is equal to the upper left corner of x=width() and y=height() if
+ // such a cell would exist.
+ if (x < 0 || (unsigned int)(x + 1) >= width()) {
+	return 1.0f;
+ }
+ if (y < 0 || (unsigned int)(y + 1) >= height()) {
+	return 1.0f;
+ }
+ return mHeightMap[y * (width() + 1) + x];
+}
+
+void BosonMap::setHeightAtCorner(int x, int y, float h)
+{
+ BO_CHECK_NULL_RET(mHeightMap);
+ if (x < 0 || (unsigned int)(x + 1) >= width()) {
+	return;
+ }
+ if (y < 0 || (unsigned int)(y + 1) >= height()) {
+	return;
+ }
+
+ // AB: see pixelToHeight() for explanation on these restrictions
+ h = QMIN(h, 15.0f);
+ h = QMAX(h, -10.5f);
+ mHeightMap[y * (width() + 1) + x] = h;
 }
 
 void BosonMap::slotChangeCell(int x, int y, int groundType, unsigned char b)
@@ -541,7 +646,7 @@ void BosonMap::slotChangeCell(int x, int y, int groundType, unsigned char b)
 //boDebug() << width() << " " << height() << endl;
  Cell* c = cell(x, y);
  if (!c) {
-	boError() << "Invalid cell x=" << x << ",y=" << y << endl;
+	boError() << k_funcinfo << "Invalid cell x=" << x << ",y=" << y << endl;
 	return;
  }
  if ((Cell::GroundType)groundType == Cell::GroundUnknown) {
@@ -560,7 +665,7 @@ void BosonMap::loadTiles(const QString& tiles, bool withtimer)
  connect(mTiles, SIGNAL(signalTilesLoaded()),
 		this, SIGNAL(signalTilesLoaded()));
  QString dir = KGlobal::dirs()->findResourceDir("data", QString("boson/themes/grounds/%1/index.ground").arg(tiles)) + QString("boson/themes/grounds/%1").arg(tiles);
- if (dir == QString::null) {
+ if (dir.isNull()) {
 	boError() << k_funcinfo << "Cannot find tileset " << tiles << endl;
 	return;
  }
@@ -621,3 +726,40 @@ void BosonMap::fill(int ground)
  }
 }
 
+
+float BosonMap::pixelToHeight(int p)
+{
+ // we have 255 different values available. we use 0.1 steps, i.e. if
+ // the pixel is increased by 1 the height is increased by 0.1.
+ // values below 105 cause a negative height, values above 105 a positive
+ // height.
+ // this means we have a height range from -105/10 to 150/10 aka from -10.5 to
+ // 15.0
+// boDebug() << k_funcinfo << p << "->" << ((float)(p-105))/10 << endl;
+ return ((float)(p - 105)) / 10;
+}
+
+int BosonMap::heightToPixel(float height)
+{
+ return (int)(height * 10 + 105);
+}
+
+float BosonMap::cellAverageHeight(int x, int y)
+{
+ Cell* c = cell(x, y);
+ if (!c) {
+	return 0;
+ }
+
+ float minz = 1000;
+ float maxz = -1000;
+
+ for (int i = x; i <= x + 1; i++) {
+	for (int j = y; j <= y + 1; j++) {
+		minz = QMIN(minz, heightAtCorner(i, j));
+		maxz = QMAX(maxz, heightAtCorner(i, j));
+	}
+ }
+
+ return (minz + maxz) / 2;
+}
