@@ -56,12 +56,12 @@ void BoUnitEditor::slotTypeChanged()
 {
     if(mUnitTypeFacility->isChecked()) {
 	// Type is facility
-	mUnitHarvestGroup->setEnabled(false);
+	mUnitCanHarvest->setEnabled(false);
 	mMobileOptions->setEnabled(false);
 	mFacilityOptions->setEnabled(true);
     } else {
 	// Type is mobile
-	mUnitHarvestGroup->setEnabled(true);
+	mUnitCanHarvest->setEnabled(true);
 	mMobileOptions->setEnabled(true);
 	mFacilityOptions->setEnabled(false);
     }
@@ -70,7 +70,7 @@ void BoUnitEditor::slotTypeChanged()
 void BoUnitEditor::slotUnitSelected( int index)
 {
     // Ask is user wants to save current properties
-    if(mUnitLoaded) {
+    if(mUnitLoaded && mConfigChanged) {
 	int answer = KMessageBox::questionYesNoCancel(this,
 						      i18n("Do you want to save current unit properties?"),
 						      i18n("Save current unit?"));
@@ -92,11 +92,15 @@ void BoUnitEditor::slotAddTexture()
 			    mUnitTextureTo->text());
     mUnitTextureFrom->clear();
     mUnitTextureTo->clear();
+    mConfigChanged = true;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotRemoveTexture()
 {
     delete mUnitTexturesList->currentItem();
+    mConfigChanged = true;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotCurrentTextureChanged()
@@ -119,6 +123,8 @@ void BoUnitEditor::slotAutoPickId()
 	id++;
     }
     mUnitId->setValue(id);
+    mConfigChanged = true;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotSaveUnit()
@@ -161,22 +167,28 @@ void BoUnitEditor::slotSaveUnit()
     }
     slotUpdateUnitProperties();
     mUnit->saveUnitType(d.absPath() + "/index.unit");
+    mConfigChanged = false;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotNewUnit()
 {
     // Ask is user wants to save current properties
-    int answer = KMessageBox::questionYesNoCancel(this,
-						  i18n("Do you want to save current unit properties?"),
-						  i18n("Save current unit?"));
-    if(answer == KMessageBox::Cancel) {
-	return;
-    } else if(answer == KMessageBox::Yes) {
-	slotSaveUnit();
+    if(mConfigChanged) {
+	int answer = KMessageBox::questionYesNoCancel(this,
+						      i18n("Do you want to save current unit properties?"),
+						      i18n("Save current unit?"));
+	if(answer == KMessageBox::Cancel) {
+	    return;
+	} else if(answer == KMessageBox::Yes) {
+	    slotSaveUnit();
+	}
     }
     // Clear everything to default values
     mUnit->reset();
     slotUpdateWidgets();
+    mConfigChanged = false;
+    updateConfigWidgets();
 }
 
 QStringList BoUnitEditor::verifyProperties()
@@ -191,6 +203,12 @@ QStringList BoUnitEditor::verifyProperties()
     if(mUnitHarvestMinerals->isChecked() && mUnitHarvestOil->isChecked()) {
 	errors += i18n("* Unit cannot mine *both* oil and minerals");
     }
+    if(mUnitRefineMinerals->isChecked() && mUnitRefineOil->isChecked()) {
+	errors += i18n("* Unit cannot refine *both* oil and minerals");
+    }
+    if(mUnitCanHarvest->isChecked() && mUnitCanRefine->isChecked()) {
+	errors += i18n("* Unit cannot mine *and* harvest");
+    }
     return errors;
 }
 
@@ -200,6 +218,12 @@ void BoUnitEditor::slotLoadUnit( QString dir )
     mUnit->loadUnitType(dir + "/index.unit", false);
     slotUpdateWidgets();
     mUnitLoaded = true;
+    // If unit has any weapons, make the first one selected
+    if(mUnit->canShoot()) {
+	mWeaponsList->setSelected(0, true);
+    }
+    mConfigChanged = false;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotEditSearchPaths()
@@ -232,13 +256,21 @@ void BoUnitEditor::init()
     mUnitProducerList->setValidator(v);
     mUnitRequirements->setValidator(v);
     mUnitDestroyedParticles->setValidator(v);
+    mUnitConstructedParticles->setValidator(v);
     mWeaponShootParticles->setValidator(v);
     mWeaponFlyParticles->setValidator(v);
     mWeaponHitParticles->setValidator(v);
     
     // Clear everything to default values
-    mUnit->reset();
+    if(mUnits.isEmpty()) {
+	mUnit->reset();
+    } else {
+	mUnitsList->setSelected(0, true);
+	//slotLoadUnit(mUnits[0]);
+    }
     slotUpdateWidgets();
+    mConfigChanged = false;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotHideSearchPaths()
@@ -298,13 +330,15 @@ void BoUnitEditor::destroy()
 void BoUnitEditor::slotOpenUnit()
 {
     // Ask is user wants to save current properties
-    int answer = KMessageBox::questionYesNoCancel(this,
-						  i18n("Do you want to save current unit properties?"),
-						  i18n("Save current unit?"));
-    if(answer == KMessageBox::Cancel) {
-	return;
-    } else if(answer == KMessageBox::Yes) {
-	slotSaveUnit();
+    if(mConfigChanged) {
+	int answer = KMessageBox::questionYesNoCancel(this,
+						      i18n("Do you want to save current unit properties?"),
+						      i18n("Save current unit?"));
+	if(answer == KMessageBox::Cancel) {
+	    return;
+	} else if(answer == KMessageBox::Yes) {
+	    slotSaveUnit();
+	}
     }
     
     // Open new unit
@@ -339,15 +373,13 @@ void BoUnitEditor::slotUpdateUnitProperties()
     // General page
     mUnit->setName(mUnitName->text());
     mUnit->setId(mUnitId->value());
-    mUnit->setUnitWidth(mUnitWidth->value() * BO_TILE_SIZE);
-    mUnit->setUnitHeight(mUnitHeight->value() * BO_TILE_SIZE);
-    mUnit->setUnitDepth(mUnitDepth->value() * BO_TILE_SIZE);
+    mUnit->setUnitWidth((unsigned int)mUnitWidth->value() * BO_TILE_SIZE);
+    mUnit->setUnitHeight((unsigned int)mUnitHeight->value() * BO_TILE_SIZE);
+    mUnit->setUnitDepth((unsigned int)mUnitDepth->value() * BO_TILE_SIZE);
     
     // Mobile/facility properties
     if(mUnitTypeFacility->isChecked()) {
 	mUnit->createFacilityProperties();
-	mUnit->setCanRefineMinerals(mUnitCanRefineMinerals->isChecked());
-	mUnit->setCanRefineOil(mUnitCanRefineOil->isChecked());
 	mUnit->setConstructionSteps(mUnitConstructionSteps->value());
     } else {
 	mUnit->createMobileProperties();
@@ -363,7 +395,6 @@ void BoUnitEditor::slotUpdateUnitProperties()
     mUnit->setOilCost(mUnitOilCost->value());
     mUnit->setSightRange(mUnitSight->value());
     mUnit->setTerrainType((UnitProperties::TerrainType)(mUnitTerrain->currentItem()));
-    mUnit->setDestroyedParticleSystemIds(stringToList(mUnitDestroyedParticles->text()));
     mUnit->setSupportMiniMap(mUnitSupportMiniMap->isChecked());
     // Weapons page
     // Sync current weapon first
@@ -384,6 +415,14 @@ void BoUnitEditor::slotUpdateUnitProperties()
 	p->setCanMineMinerals(mUnitHarvestMinerals->isChecked());
 	p->setCanMineOil(mUnitHarvestOil->isChecked());
 	p->setMaxResources(mUnitMaxResource->value());
+	p->setMiningSpeed(mUnitMiningSpeed->value());
+	p->setUnloadingSpeed(mUnitUnloadingSpeed->value());
+	mUnit->addPlugin(p);
+    }
+    if(mUnitCanRefine->isChecked()) {
+	RefineryProperties* p = new RefineryProperties(mUnit);
+	p->setCanRefineMinerals(mUnitRefineMinerals->isChecked());
+	p->setCanRefineOil(mUnitRefineOil->isChecked());
 	mUnit->addPlugin(p);
     }
     if(mUnitCanRepair->isChecked()) {
@@ -402,13 +441,20 @@ void BoUnitEditor::slotUpdateUnitProperties()
 	    mUnit->addTextureMapping(it.current()->text(0), it.current()->text(1));
 	}
     }
-    mUnit->addSound(SoundShoot, mUnitSoundShoot->text());
     mUnit->addSound(SoundOrderMove, mUnitSoundOrderMove->text());
     mUnit->addSound(SoundOrderAttack, mUnitSoundOrderAttack->text());
     mUnit->addSound(SoundOrderSelect, mUnitSoundOrderSelect->text());
     mUnit->addSound(SoundReportProduced, mUnitSoundReportProduced->text());
     mUnit->addSound(SoundReportDestroyed, mUnitSoundReportDestroyed->text());
     mUnit->addSound(SoundReportUnderAttack, mUnitSoundReportUnderAttack->text());
+    // Other page
+    mUnit->setDestroyedParticleSystemIds(stringToList(mUnitDestroyedParticles->text()));
+    mUnit->setConstructedParticleSystemIds(stringToList(mUnitConstructedParticles->text()));
+    mUnit->setExplodingDamageRange(mUnitExplodingDamageRange->value());
+    mUnit->setExplodingDamage(mUnitExplodingDamage->value());
+    BoVector3 hitpoint(mUnitHitPointX->value(), mUnitHitPointY->value(), mUnitHitPointZ->value());
+    hitpoint.cellToCanvas();
+    mUnit->setHitPoint(hitpoint);
 }
 
 
@@ -434,7 +480,6 @@ void BoUnitEditor::slotUpdateWidgets()
     mUnitSight->setValue(mUnit->sightRange());
     int terrain = (int)(mUnit->terrainType());
     mUnitTerrain->setCurrentItem(terrain);
-    mUnitDestroyedParticles->setText(listToString(mUnit->destroyedParticleSystemIds()));
     mUnitSupportMiniMap->setChecked(mUnit->supportMiniMap());
     // FIXME: UnitProperties only saves mobile *or* facility properties, but
     //  I'd like to have them both saved
@@ -442,8 +487,6 @@ void BoUnitEditor::slotUpdateWidgets()
     mUnitSpeed->setValue(mUnit->speed());
     mUnitCanGoOnLand->setChecked(mUnit->canGoOnLand());
     mUnitCanGoOnWater->setChecked(mUnit->canGoOnWater());
-    mUnitCanRefineMinerals->setChecked(mUnit->canRefineMinerals());
-    mUnitCanRefineOil->setChecked(mUnit->canRefineOil());
     mUnitConstructionSteps->setValue(mUnit->constructionSteps());
     // Weapons
     mWeapons.clear();
@@ -477,6 +520,16 @@ void BoUnitEditor::slotUpdateWidgets()
 	mUnitHarvestMinerals->setChecked(harvesterProp->canMineMinerals());
 	mUnitHarvestOil->setChecked(harvesterProp->canMineOil());
 	mUnitMaxResource->setValue(harvesterProp->maxResources());
+	mUnitMiningSpeed->setValue(harvesterProp->miningSpeed());
+	mUnitUnloadingSpeed->setValue(harvesterProp->unloadingSpeed());
+    }
+    const RefineryProperties* refineryProp = (RefineryProperties*)(mUnit->properties(PluginProperties::Refinery));
+    bool canRefine = (refineryProp != 0l);
+    mUnitCanRefine->setChecked(canRefine);
+    mUnitRefineGroup->setEnabled(canRefine);
+    if(canRefine) {
+	mUnitRefineMinerals->setChecked(refineryProp->canRefineMinerals());
+	mUnitRefineOil->setChecked(refineryProp->canRefineOil());
     }
     bool canRepair = (mUnit->properties(PluginProperties::Repair) != 0l);
     mUnitCanRepair->setChecked(canRepair);
@@ -491,13 +544,22 @@ void BoUnitEditor::slotUpdateWidgets()
     for(tit = textures.begin(); tit != textures.end(); ++tit) {
 	(void)new QListViewItem(mUnitTexturesList, tit.key(), tit.data());
     }
-    mUnitSoundShoot->setText(mUnit->sound(SoundShoot));
     mUnitSoundOrderMove->setText(mUnit->sound(SoundOrderMove));
     mUnitSoundOrderAttack->setText(mUnit->sound(SoundOrderAttack));
     mUnitSoundOrderSelect->setText(mUnit->sound(SoundOrderSelect));
     mUnitSoundReportProduced->setText(mUnit->sound(SoundReportProduced));
     mUnitSoundReportDestroyed->setText(mUnit->sound(SoundReportDestroyed));
     mUnitSoundReportUnderAttack->setText(mUnit->sound(SoundReportUnderAttack));
+    // Other page
+    mUnitDestroyedParticles->setText(listToString(mUnit->destroyedParticleSystemIds()));
+    mUnitConstructedParticles->setText(listToString(mUnit->constructedParticleSystemIds()));
+    mUnitExplodingDamageRange->setValue(mUnit->explodingDamageRange());
+    mUnitExplodingDamage->setValue(mUnit->explodingDamage());
+    BoVector3 hitpoint = mUnit->hitPoint();
+    hitpoint.canvasToCell();
+    mUnitHitPointX->setValue(hitpoint.x());
+    mUnitHitPointY->setValue(hitpoint.y());
+    mUnitHitPointZ->setValue(hitpoint.z());
 }
 
 void BoUnitEditor::slotUpdateWeaponProps()
@@ -509,39 +571,48 @@ void BoUnitEditor::slotUpdateWeaponProps()
     w->setWeaponName(mWeaponName->text());
     w->setDamage(mWeaponDamage->value());
     w->setDamageRange(mWeaponDamageRange->value());
+    w->setFullDamageRange(mWeaponFullDamageRange->value());
     w->setReloadingTime(mWeaponReload->value());
     w->setRange(mWeaponRange->value());
     w->setCanShootAtAirUnits(mWeaponCanShootAtAirUnits->isChecked());
     w->setCanShootAtLandUnits(mWeaponCanShootAtLandUnits->isChecked());
-    w->setSpeed(mWeaponSpeed->value());
+    w->setSpeed((long unsigned int)mWeaponSpeed->value());
     w->setModelFileName(mWeaponModel->text());
-    w->setMaxHeight(mWeaponMaxHeight->value());
+    w->setHeight(mWeaponHeight->value());
     BoVector3 offset(mWeaponOffsetX->value(), mWeaponOffsetY->value(), mWeaponOffsetZ->value());
-    offset.scale(BO_TILE_SIZE);
+    offset.cellToCanvas();
     w->setOffset(offset);
     w->setShootParticleSystemIds(stringToList(mWeaponShootParticles->text()));
     w->setFlyParticleSystemIds(stringToList(mWeaponFlyParticles->text()));
     w->setHitParticleSystemIds(stringToList(mWeaponHitParticles->text()));
+    w->setSound(SoundWeaponShoot, mWeaponSoundShoot->text());
 }
 
 
 void BoUnitEditor::slotAddWeapon()
 {
-    BosonWeaponProperties* w = new BosonWeaponProperties(mUnit);
+    // 0 is actually invalid id... but we don't care as it's only used for loading/saving games
+    BosonWeaponProperties* w = new BosonWeaponProperties(mUnit, 0);
     w->reset();
     mWeapons.append(w);
     mWeaponsList->insertItem("New weapon", mWeapons.count() - 1);
     mWeaponsList->setCurrentItem(mWeapons.count() - 1);
     slotWeaponSelected(mWeapons.count() - 1);
+    mConfigChanged = true;
+    updateConfigWidgets();
 }
 
 void BoUnitEditor::slotWeaponSelected( int index )
 {
+    // Selecting another weapon modifies widget's contents, but we want to maintain old configChanged status
+    bool configWasChanged = mConfigChanged;
     if(mCurrentWeapon != -1) {
 	slotUpdateWeaponProps();
     }
     mCurrentWeapon = index;
     slotUpdateWeaponWidgets();
+    mConfigChanged = configWasChanged;
+    updateConfigWidgets();
 }
 
 
@@ -558,6 +629,8 @@ void BoUnitEditor::slotRemoveWeapon()
 	mCurrentWeapon--;
     }
     slotUpdateWeaponWidgets();
+    mConfigChanged = true;
+    updateConfigWidgets();
 }
 
 
@@ -574,18 +647,35 @@ void BoUnitEditor::slotUpdateWeaponWidgets()
     mWeaponName->setText(w->weaponName());
     mWeaponDamage->setValue(w->damage());
     mWeaponDamageRange->setValue(w->damageRange());
+    mWeaponFullDamageRange->setValue(w->fullDamageRange());
     mWeaponReload->setValue(w->reloadingTime());
     mWeaponRange->setValue(w->range());
     mWeaponCanShootAtAirUnits->setChecked(w->canShootAtAirUnits());
     mWeaponCanShootAtLandUnits->setChecked(w->canShootAtLandUnits());
     mWeaponSpeed->setValue(w->speed());
     mWeaponModel->setText(w->modelFileName());
-    mWeaponMaxHeight->setValue(w->maxHeight());
+    mWeaponHeight->setValue(w->height());
     BoVector3 o = w->offset();
-    mWeaponOffsetX->setValue(o[0] / BO_TILE_SIZE);
-    mWeaponOffsetY->setValue(o[1] / BO_TILE_SIZE);
-    mWeaponOffsetZ->setValue(o[2] / BO_TILE_SIZE);
+    o.canvasToCell();
+    mWeaponOffsetX->setValue(o.x());
+    mWeaponOffsetY->setValue(o.y());
+    mWeaponOffsetZ->setValue(o.z());
     mWeaponShootParticles->setText(listToString(w->shootParticleSystemIds()));
     mWeaponFlyParticles->setText(listToString(w->flyParticleSystemIds()));
     mWeaponHitParticles->setText(listToString(w->hitParticleSystemIds()));
+    mWeaponSoundShoot->setText(w->sound(SoundWeaponShoot));
+}
+
+
+void BoUnitEditor::slotConfigChanged()
+{
+    // Unit config has changed
+    mConfigChanged = true;
+    updateConfigWidgets();
+}
+
+
+void BoUnitEditor::updateConfigWidgets()
+{
+    mUnitSaveButton->setEnabled(mConfigChanged);
 }
