@@ -24,6 +24,7 @@
 #include "speciestheme.h"
 #include "unitproperties.h"
 #include "bo3dtools.h"
+#include "bo3dsload.h"
 #include "bodebug.h"
 
 #include <qcombobox.h>
@@ -37,11 +38,16 @@
 #include <qvgroupbox.h>
 #include <qgrid.h>
 #include <qheader.h>
+#include <qsplitter.h>
+#include <qvbox.h>
+#include <qstringlist.h>
+#include <ksimpleconfig.h>
 
 #include <klistbox.h>
 #include <klistview.h>
 #include <klocale.h>
 #include <kpopupmenu.h>
+#include <kstandarddirs.h>
 
 #include <lib3ds/file.h>
 #include <lib3ds/node.h>
@@ -404,6 +410,9 @@ public:
 		mUnconnectedFacesList = 0;
 		mMeshMatrix = 0;
 
+		mFacesCountLabel = 0;
+		mVertexCountLabel = 0;
+
 		m3ds = 0;
 	}
 
@@ -428,6 +437,9 @@ public:
 	BoFaceView* mUnconnectedFacesList;
 	BoMatrixWidget* mMeshMatrix;
 
+	QLabel* mFacesCountLabel;
+	QLabel* mVertexCountLabel;
+
 	int mCurrentItem;
 	Lib3dsFile* m3ds;
 };
@@ -470,16 +482,16 @@ void KGameModelDebug::initMaterialPage()
 {
  d->mMaterialPage = new QWidget(d->mTabWidget);
  QHBoxLayout* l = new QHBoxLayout(d->mMaterialPage, 10, 10);
- d->mMaterialBox = new KListBox(d->mMaterialPage);
+ QSplitter* splitter = new QSplitter(d->mMaterialPage);
+ l->addWidget(splitter, 0);
+
+ d->mMaterialBox = new KListBox(splitter);
  connect(d->mMaterialBox, SIGNAL(executed(QListBoxItem*)), this, SLOT(slotDisplayMaterial(QListBoxItem*)));
- l->addWidget(d->mMaterialBox, 0);
  QFontMetrics metrics(font());
 
- d->mMaterialData = new BoMaterialWidget(d->mMaterialPage);
- l->addWidget(d->mMaterialData, 1);
+ d->mMaterialData = new BoMaterialWidget(splitter);
 
- d->mTextureView = new KListView(d->mMaterialPage);
- l->addWidget(d->mTextureView, 1);
+ d->mTextureView = new KListView(splitter);
  d->mTextureView->addColumn(i18n("Map"));
  d->mTextureView->addColumn(i18n("Name"));
  d->mTextureView->addColumn(i18n("Flags"), metrics.width(QString::number(111)));
@@ -502,39 +514,37 @@ void KGameModelDebug::initMeshPage()
 {
  d->mMeshPage = new QWidget(d->mTabWidget);
  QHBoxLayout* l = new QHBoxLayout(d->mMeshPage, 10, 10);
+ QSplitter* splitter = new QSplitter(d->mMeshPage);
+ l->addWidget(splitter);
  QFontMetrics metrics(font());
 
- QVBoxLayout* meshViewLayout = new QVBoxLayout();
- l->addLayout(meshViewLayout, 0);
- d->mMeshView = new KListView(d->mMeshPage);
+ QVBox* meshView = new QVBox(splitter);
+ d->mMeshView = new KListView(meshView);
  d->mMeshView->addColumn(i18n("Name"), metrics.width(i18n("Name")));
  d->mMeshView->addColumn(i18n("Color"), metrics.width(QString::number(111)));
  d->mMeshView->addColumn(i18n("Points count"), metrics.width(QString::number(111)));
  d->mMeshView->addColumn(i18n("Texels count"), metrics.width(QString::number(111)));
  d->mMeshView->addColumn(i18n("Faces count"), metrics.width(QString::number(111)));
  d->mMeshView->addColumn(i18n("Flags count"), metrics.width(QString::number(11)));
+ d->mMeshView->addColumn(i18n("Max point index"), metrics.width(QString::number(11)));
  connect(d->mMeshView, SIGNAL(executed(QListViewItem*)), this, SLOT(slotDisplayMesh(QListViewItem*)));
- meshViewLayout->addWidget(d->mMeshView);
- d->mMeshView->resize(d->mMeshView->height(), 150);
+ QVBox* modelInfo = new QVBox(meshView); // actually it doesn't fit to "meshView", but rather display info about the model
+ QHBox* faces = new QHBox(modelInfo);
+ (void)new QLabel(i18n("Faces: "), faces);
+ d->mFacesCountLabel = new QLabel(faces);
+ QHBox* vertices = new QHBox(modelInfo);
+ (void)new QLabel(i18n("Vertices (Faces * 3): "), vertices);
+ d->mVertexCountLabel = new QLabel(vertices);
 
- QVBoxLayout* faceLayout = new QVBoxLayout();
- l->addLayout(faceLayout, 1);
- d->mFaceList = new BoFaceView(d->mMeshPage);
+ QVBox* faceView = new QVBox(splitter);
+ d->mFaceList = new BoFaceView(faceView);
  connect(d->mFaceList, SIGNAL(executed(QListViewItem*)), this, SLOT(slotConnectToFace(QListViewItem*)));
- QLabel* connectableLabel = new QLabel(i18n("Connectable to selected face:"), d->mMeshPage);
- d->mConnectedFacesList = new BoFaceView(d->mMeshPage);
- QLabel* unconnectableLabel = new QLabel(i18n("Unconnectable to selected face:"), d->mMeshPage);
- d->mUnconnectedFacesList = new BoFaceView(d->mMeshPage);
- faceLayout->addWidget(d->mFaceList);
- faceLayout->addWidget(connectableLabel);
- faceLayout->addWidget(d->mConnectedFacesList);
- faceLayout->addWidget(unconnectableLabel);
- faceLayout->addWidget(d->mUnconnectedFacesList);
+ (void)new QLabel(i18n("Connectable to selected face:"), faceView);
+ d->mConnectedFacesList = new BoFaceView(faceView);
+ (void)new QLabel(i18n("Unconnectable to selected face:"), faceView);
+ d->mUnconnectedFacesList = new BoFaceView(faceView);
 
- QVBoxLayout* meshDataLayout = new QVBoxLayout();
- l->addLayout(meshDataLayout);
- QVGroupBox* meshMatrixBox = new QVGroupBox(i18n("Matrix"), d->mMeshPage);
- meshDataLayout->addWidget(meshMatrixBox);
+ QVGroupBox* meshMatrixBox = new QVGroupBox(i18n("Matrix"), splitter);
  d->mMeshMatrix = new BoMatrixWidget(meshMatrixBox);
 
  d->mTabWidget->addTab(d->mMeshPage, i18n("&Meshes"));
@@ -560,6 +570,27 @@ void KGameModelDebug::addTheme(SpeciesTheme* theme)
 	QString file = (*it)->unitPath() + SpeciesTheme::unitModelFile();
 	addModel(file, (*it)->name());
  }
+ 
+ // FIXME: duplicated code
+ QString fileName = theme->themePath() + QString::fromLatin1("objects/objects.boson");
+ if (!KStandardDirs::exists(fileName)) {
+	boDebug() << k_funcinfo << "no objects.boson file found" << endl;
+	// We assume that this theme has no objects and don't complain
+	return;
+ }
+
+ KSimpleConfig cfg(fileName);
+ QStringList objects = cfg.groupList();
+ QStringList::Iterator oit;
+ for (oit = objects.begin(); oit != objects.end(); ++oit) {
+	cfg.setGroup(*oit);
+	float width, height;
+	QString file;
+	width = (float)cfg.readDoubleNumEntry("Width", 1.0);
+	height = (float)cfg.readDoubleNumEntry("Height", 1.0);
+	file = cfg.readEntry("File", "missile.3ds");
+	addModel(theme->themePath() + QString::fromLatin1("objects/") + file, *oit);
+ }
 }
 
 void KGameModelDebug::slotModelChanged(int index)
@@ -590,6 +621,20 @@ void KGameModelDebug::slotUpdate()
 
  updateMaterialPage();
  updateMeshPage();
+
+ if (!d->m3ds) {
+	return;
+ }
+ int faces = 0;
+ Lib3dsMesh* mesh = d->m3ds->meshes;
+ for (; mesh; mesh = mesh->next) {
+	faces += mesh->faces;
+ }
+ d->mFacesCountLabel->setText(QString::number(faces));
+ d->mVertexCountLabel->setText(QString::number(faces * 3));
+ //TODO: we could also display how many points will actually get rendered. could
+ //be interesting in case we ever use triangle strips. currently those values
+ //would be exactly the same.
 }
 
 void KGameModelDebug::updateMaterialPage()
@@ -640,6 +685,14 @@ void KGameModelDebug::slotConstructMeshList()
 	item->setText(3, QString::number(mesh->texels));
 	item->setText(4, QString::number(mesh->faces));
 	item->setText(5, QString::number(mesh->flags));
+	int indices = 0;
+	for (unsigned int i = 0; i < mesh->faces; i++) {
+		Lib3dsFace* f = &mesh->faceL[i];
+		indices = QMAX(indices, f->points[0]);
+		indices = QMAX(indices, f->points[1]);
+		indices = QMAX(indices, f->points[2]);
+	}
+	item->setText(6, QString::number(indices));
 	d->mListItem2Mesh.insert(item, mesh);
  }
 }
@@ -747,7 +800,7 @@ void KGameModelDebug::slotConnectToFace(QListViewItem* item)
 	boError() << k_funcinfo << "NULL mesh" << endl;
 	return;
  }
- BosonModel::findAdjacentFaces(&connected, mesh, face);
+ Bo3DSLoad::findAdjacentFaces(&connected, mesh, face);
  QPtrList<Lib3dsFace> faces;
  for (unsigned int i = 0; i < mesh->faces; i++) {
 	Lib3dsFace* f = &mesh->faceL[i];
