@@ -27,13 +27,17 @@
 #include "bosonstatistics.h"
 #include "boson.h"
 #include "upgradeproperties.h"
+#include "unitpropertyhandler.h"
+#include "bosonpropertyxml.h"
+#include "bosonprofiling.h"
 #include "bodebug.h"
 
-#include <kgame/kgamepropertyhandler.h>
 #include <kgame/kgame.h>
 #include <kgame/kgamemessage.h>
 
 #include <qbitarray.h>
+#include <qdom.h>
+#include <qtextstream.h>
 
 #include "player.moc"
 
@@ -72,6 +76,13 @@ Player::Player() : KPlayer()
  setAsyncInput(true);
  connect(this, SIGNAL(signalNetworkData(int, const QByteArray&, Q_UINT32, KPlayer*)),
 		this, SLOT(slotNetworkData(int, const QByteArray&, Q_UINT32, KPlayer*)));
+
+ KGamePropertyBase* propName = dataHandler()->find(KGamePropertyBase::IdName);
+ if (propName) {
+	propName->setPolicy(KGamePropertyBase::PolicyClean);
+ } else {
+	boError() << k_funcinfo << "can't find name property" << endl;
+ }
 // TODO d->mFogged.registerData() or something like this
  d->mMinerals.registerData(IdMinerals, dataHandler(),
 		KGamePropertyBase::PolicyLocal, "MineralCost");
@@ -125,7 +136,7 @@ void Player::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 sende
  }
 // first check if the message is a property of a unit
  QPtrListIterator<Unit> it(d->mUnits);
- while(it.current() && !it.current()->dataHandler()->processMessage(stream, msgid + KGamePropertyBase::IdUser, issender)) {
+ while (it.current() && !it.current()->dataHandler()->processMessage(stream, msgid + KGamePropertyBase::IdUser, issender)) {
 	++it;
  }
  Unit* unit = it.current();
@@ -161,7 +172,8 @@ void Player::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 sende
 
  // this wasn't a unit property but a normal message
  switch (msgid) {
-	// nothing done here currently
+	case BosonMessage::IdChat:
+		break;
 	default:
 		boWarning() << k_funcinfo << "Unknown message " << msgid << endl;
 		break;
@@ -182,14 +194,11 @@ void Player::addUnit(Unit* unit)
  unit->setOwner(this); // already done in c'tor of Unit
  unit->dataHandler()->registerHandler(BosonMessage::UnitPropertyHandler + d->mUnitPropID, this,
 		SLOT(sendProperty(int, QDataStream&, bool*)),
-		SIGNAL(signalUnitPropertyChanged(KGamePropertyBase*)));
- connect(unit->dataHandler(), SIGNAL(signalPropertyChanged(KGamePropertyBase*)), 
-		this, SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
+		SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
 
  if (unit->isMobile()) {
 	d->mMobilesCount++;
- }
- else {
+ } else {
 	d->mFacilitiesCount++;
  }
 }
@@ -208,9 +217,11 @@ void Player::unitDestroyed(Unit* unit)
 	statistics()->addLostMobileUnit(unit);
 	d->mMobilesCount--;
  }
- if (!hasMiniMap()) {
-	speciesTheme()->playSound(SoundReportMinimapDeactivated);
-	emit signalShowMiniMap(false);
+ if (unit->unitProperties()->supportMiniMap()) {
+	if (!hasMiniMap()) {
+		speciesTheme()->playSound(SoundReportMinimapDeactivated);
+		emit signalShowMiniMap(false);
+	}
  }
 }
 
@@ -221,25 +232,8 @@ void Player::slotUnitPropertyChanged(KGamePropertyBase* prop)
 	return;
  }
 
-// VERY EVIL HACK!!!
- Unit* unit;
- bool found = false;
- for (unit = d->mUnits.first(); unit && !found; unit = d->mUnits.next()) {
-	if (unit->dataHandler() == (KGamePropertyHandler*)sender()) {
-		found = true;
-//		boDebug() << "found unit " << unit->id() << endl;
-		break;
-	}
- }
-// (evil hack end)
-
- if (!unit) {
-	boError() << k_funcinfo << "NULL unit" << endl;
-	boDebug() << "player=" << id() << ",propId=" << prop->id() << ",units=" << d->mUnits.count() << endl;
-	return;
- }
-
- switch(prop->id()) {
+ bool emitSignalUnitChanged = false;
+ switch (prop->id()) {
 	case UnitBase::IdHealth:
 	case UnitBase::IdArmor:
 	case UnitBase::IdShields:
@@ -247,13 +241,33 @@ void Player::slotUnitPropertyChanged(KGamePropertyBase* prop)
 		// update BosonUnitView if the unit is selected.
 		// not all of these IDs are displayed there. But perhaps they
 		// will one day.
-		emit signalUnitChanged(unit);
+		emitSignalUnitChanged = true;
 		break;
 	default:
 		// all other Unit IDs are not displayed in BosonUnitView so
 		// there is no need to emit a signal for them.
 		break;
  }
+ if (!emitSignalUnitChanged) {
+	// nothing to do here
+	return;
+ }
+
+ // AB: we don't check for sender()->isA() here, for performance reasons.
+ UnitPropertyHandler* p = (UnitPropertyHandler*)sender();
+ if (!p->unit()) {
+	boError() << k_funcinfo << "NULL parent unit for property handler" << endl;
+	boDebug() << "player=" << id() << ",propId=" << prop->id() << endl;
+	return;
+ }
+ Unit* unit = (Unit*)p->unit();
+ if (!unit) {
+	boError() << k_funcinfo << "NULL unit" << endl;
+	return;
+ }
+ if (emitSignalUnitChanged) {
+	emit signalUnitChanged(unit);
+ }
 }
 
 Unit* Player::findUnit(unsigned long int unitId) const
@@ -291,11 +305,15 @@ bool Player::save(QDataStream& stream)
  Q_UINT32 unitPropId = d->mUnitPropID;
  stream << unitPropId;
 
+
+ // AB: see load()
+ /*
  // Save fog
  stream << d->mFogged;
 
  // Save statistics
  d->mStatistics->save(stream);
+ */
 
  /// TODO: save researched technologies!!!
 
@@ -312,8 +330,6 @@ bool Player::saveUnits(QDataStream& stream)
 	// we need the type first!
 	stream << (unsigned long int)unit->type();
 	stream << (unsigned long int)unit->id();
-	stream << (double)unit->x();
-	stream << (double)unit->y();
 
 	stream << (int)unit->dataHandler()->id();
 
@@ -350,11 +366,17 @@ bool Player::load(QDataStream& stream)
  stream >> unitPropId;
  d->mUnitPropID = unitPropId;
 
+ // these are necessary for loading games only. for that we use loadFromXML()
+ // now. note that Player::load() still gets called when a player enters a
+ // network game - we may want to fix this in order to enable players to join an
+ // already running game!
+ /*
  // Load fog
  stream >> d->mFogged;
 
- // Save statistics
+ // Load statistics
  d->mStatistics->load(stream);
+ */
 
  return true;
 }
@@ -367,12 +389,9 @@ bool Player::loadUnits(QDataStream& stream)
  for (unsigned int i = 0; i < unitCount; i++) {
 	unsigned long int type;
 	unsigned long int id;
-	double x, y;
 	int dataHandlerID;
 	stream >> type;
 	stream >> id;
-	stream >> x;
-	stream >> y;
 	stream >> dataHandlerID;
 
 	// Create unit with Boson
@@ -388,13 +407,9 @@ bool Player::loadUnits(QDataStream& stream)
 	d->mUnits.append(unit);
 	unit->dataHandler()->registerHandler(dataHandlerID, this,
 			SLOT(sendProperty(int, QDataStream&, bool*)),
-			SIGNAL(signalUnitPropertyChanged(KGamePropertyBase*)));
-	connect(unit->dataHandler(), SIGNAL(signalPropertyChanged(KGamePropertyBase*)),
-			this, SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
+			SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
 	unit->setId(id);
 
-	// Emit signal for canvas and minimap and BosonWidget
-	emit signalUnitLoaded(unit, (int)x, (int)y);
 	// Call unit's loading methods
 	if (!unit->load(stream)) {
 		boError() << k_funcinfo << "Error while loading unit with id=" << id << endl;
@@ -411,9 +426,9 @@ bool Player::loadUnits(QDataStream& stream)
  return true;
 }
 
-QPtrList<Unit> Player::allUnits() const
+QPtrList<Unit>* Player::allUnits() const
 {
- return d->mUnits;
+ return &(d->mUnits);
 }
 
 const UnitProperties* Player::unitProperties(unsigned long int unitType) const
@@ -632,7 +647,7 @@ bool Player::hasUnitWithType(unsigned long int type) const
 
 bool Player::hasTechnology(unsigned long int id) const
 {
- TechnologyProperties* tech = speciesTheme()->technologyList().find(id);
+ UpgradeProperties* tech = speciesTheme()->technologyList().find(id);
  if (!tech) {
 	return false;
  }
@@ -646,36 +661,21 @@ void Player::technologyResearched(ProductionPlugin*, unsigned long int id)
 {
  boDebug() << k_funcinfo << "id: " << id << endl;
  // Check if it isn't researched already
- QIntDictIterator<TechnologyProperties> it(speciesTheme()->technologyList());
+ QIntDictIterator<UpgradeProperties> it(speciesTheme()->technologyList());
  while (it.current()) {
 	if (((unsigned long int)(it.currentKey()) == id) && (it.current()->isResearched())) {
-		boError() << k_funcinfo << "Technology " << it.current() << " already researched!" << endl;
+		boError() << k_funcinfo << "upgrade " << it.current() << " already researched!" << endl;
 		return;
 	}
 	++it;
  }
 
- TechnologyProperties* prop = speciesTheme()->technology(id);
+ UpgradeProperties* prop = speciesTheme()->technology(id);
  prop->setResearched(true);
  prop->apply(this);
 
- // Iterate through upgrades, applying as needed
- QValueList<unsigned long int> unitIds = speciesTheme()->allFacilities();  // FIXME: this is dirty
- unitIds += speciesTheme()->allMobiles();
- QValueList<unsigned long int>::iterator pit;
- for (pit = unitIds.begin(); pit != unitIds.end(); pit++) {
-	UpgradeProperties* upgrade;
-	QPtrList<UpgradeProperties> ulist = speciesTheme()->nonConstUnitProperties(*pit)->unresearchedUpgrades();
-	for (upgrade = ulist.first(); upgrade; upgrade = ulist.next()) {
-		if (upgrade->canBeResearched(this)) {
-			boDebug() << k_funcinfo << "    Applying upgrade for UP " << *pit << " with id " << upgrade->id() << endl;
-			upgrade->setResearched(true);
-			upgrade->apply(this);
-			speciesTheme()->nonConstUnitProperties(*pit)->upgradeResearched( upgrade);
-		}
-	}
- }
  ((Boson*)game())->slotUpdateProductionOptions();
+ // TODO: also update unit view
 }
 
 bool Player::advanceFlag() const
@@ -683,3 +683,275 @@ bool Player::advanceFlag() const
  return ((Boson*)game())->advanceFlag();
 }
 
+bool Player::saveAsXML(QDomElement& root)
+{
+ BosonProfiler profiler(BosonProfiling::SavePlayerToXML);
+
+ // AB: probably we'll never use this... (KGame should take care of the id)
+ root.setAttribute(QString::fromLatin1("Id"), id());
+
+ root.setAttribute(QString::fromLatin1("NetworkPriority"), networkPriority());
+
+ QDomDocument doc = root.ownerDocument();
+
+ // store the dataHandler()
+ BosonCustomPropertyXML propertyXML;
+ QDomElement handler = doc.createElement(QString::fromLatin1("DataHandler"));
+ if (!propertyXML.saveAsXML(handler, dataHandler())) {
+	boError() << k_funcinfo << "Unable to save datahandler of player " << id() << endl;
+	return false;
+ }
+ root.appendChild(handler);
+
+ // save units
+ QDomElement units = doc.createElement(QString::fromLatin1("Units"));
+ QPtrListIterator<Unit> it(d->mUnits);
+ for (; it.current(); ++it) {
+	Unit* u = it.current();
+	QDomElement unit = doc.createElement(QString::fromLatin1("Unit"));
+	if (!u->saveAsXML(unit)) {
+		boError() << k_funcinfo << "Could not save unit " << u->id() << endl;
+		continue;
+	}
+	units.appendChild(unit);
+ }
+ root.appendChild(units);
+
+ // Save speciestheme
+ if (speciesTheme()) {
+	root.setAttribute(QString::fromLatin1("SpeciesTheme"), speciesTheme()->identifier());
+	root.setAttribute(QString::fromLatin1("TeamColor"), (unsigned int)speciesTheme()->teamColor().rgb());
+ } else {
+	// the attributes won't be there
+	boWarning() << k_funcinfo << "NULL speciestheme while saving into XML" << endl;
+ }
+
+ // Save unitpropID
+ root.setAttribute(QString::fromLatin1("UnitPropId"), (unsigned int)d->mUnitPropID);
+
+ // Save fog
+ saveFogOfWar(root);
+
+ // Save statistics
+ QDomElement statistics = doc.createElement(QString::fromLatin1("Statistics"));
+ d->mStatistics->save(statistics);
+ root.appendChild(statistics);
+
+ return true;
+}
+
+bool Player::loadFromXML(const QDomElement& root)
+{
+ boDebug(260) << k_funcinfo << endl;
+ // this does NOT load the units!
+
+ bool ok = false;
+ int id = root.attribute(QString::fromLatin1("Id")).toInt(&ok);
+ if (!ok) {
+	boError(260) << k_funcinfo << "Id was no valid number" << endl;
+	return false;
+ }
+ Q_UNUSED(id);
+// setId(id); // AB: KGame should take care of this. we should not need this. (remember that we depend on KGame::d->mUniquePlayerNumber to be in sync!)
+ int networkPriority = root.attribute(QString::fromLatin1("NetworkPriority")).toInt(&ok);
+ if (!ok) {
+	boError(260) << k_funcinfo << "NetworkPriority was no valid number" << endl;
+	return false;
+ }
+ setNetworkPriority(networkPriority);
+
+ boDebug(260) << k_funcinfo << "load data handler" << endl;
+ // load the data handler
+ BosonCustomPropertyXML propertyXML;
+ QDomElement handler = root.namedItem(QString::fromLatin1("DataHandler")).toElement();
+ if (!propertyXML.loadFromXML(handler, dataHandler())) {
+	boError(260) << k_funcinfo << "unable to load player data handler (player=" << this->id() << ")" << endl;
+	return false;
+ }
+
+ // note: this does NOT load the units.
+
+
+ boDebug(260) << k_funcinfo << "loading speciesTheme" << endl;
+ QString speciesIdentifier;
+ QColor color;
+ if (root.hasAttribute(QString::fromLatin1("SpeciesTheme"))) {
+	speciesIdentifier = root.attribute(QString::fromLatin1("SpeciesTheme"));
+ }
+ if (root.hasAttribute(QString::fromLatin1("TeamColor"))) {
+	unsigned int c = root.attribute(QString::fromLatin1("TeamColor")).toUInt(&ok);
+	if (!ok) {
+		boError(260) << k_funcinfo << "Invalid TeamColor value" << endl;
+	} else {
+		color.setRgb(c);
+	}
+ }
+ if (speciesIdentifier.isNull()) {
+	// migh be valid if we ever use this for network loading, too!
+	boError(260) << k_funcinfo << "No SpeciesTheme" << endl;
+	return false;
+ } else {
+	boDebug(260) << k_funcinfo << "speciesTheme: " << speciesIdentifier << endl;
+	// TODO: check whether this theme actually exists and could be loaded
+	loadTheme(SpeciesTheme::speciesDirectory(speciesIdentifier), color);
+ }
+
+ d->mUnitPropID = root.attribute(QString::fromLatin1("UnitPropId")).toUInt(&ok);
+ if (!ok) {
+	boError(260) << k_funcinfo << "Invalid UnitPropId value" << endl;
+	return false;
+ }
+
+
+ // Load fog
+ boDebug(260) << k_funcinfo << "loading fow" << endl;
+ loadFogOfWar(root);
+
+ // Load statistics
+ QDomElement statistics = root.namedItem(QString::fromLatin1("Statistics")).toElement();
+ if (statistics.isNull()) {
+	boWarning() << k_funcinfo << "No Statistics tag" << endl;
+	// dont return
+ } else {
+	d->mStatistics->load(statistics);
+ }
+
+ boDebug(260) << k_funcinfo << "done" << endl;
+ return true;
+}
+
+bool Player::loadUnitsFromXML(const QDomElement& root)
+{
+ if (root.isNull()) {
+	boError(260) << k_funcinfo << "NULL root node" << endl;
+	return false;
+ }
+ QDomElement units = root.namedItem(QString::fromLatin1("Units")).toElement();
+ if (units.isNull()) {
+	boWarning(260) << k_funcinfo << "no units for player " << id() << endl;
+	return true;
+ }
+ QDomNodeList list = units.elementsByTagName(QString::fromLatin1("Unit"));
+ if (list.count() == 0) {
+	boWarning(260) << k_funcinfo << "no units for player " << id() << endl;
+	return true;
+ }
+ for (unsigned int i = 0; i < list.count(); i++) {
+	QDomElement e = list.item(i).toElement();
+	if (e.isNull()) {
+		boError(260) << k_funcinfo << i << " is not an element" << endl;
+		return false;
+	}
+	if (!e.hasAttribute(QString::fromLatin1("UnitType"))) {
+		boError(260) << k_funcinfo << "missing attribute: UnitType for Unit " << i << endl;
+		continue;
+	}
+	if (!e.hasAttribute(QString::fromLatin1("Id"))) {
+		boError(260) << k_funcinfo << "missing attribute: Id for Unit " << i << endl;
+		continue;
+	}
+	if (!e.hasAttribute(QString::fromLatin1("DataHandlerId"))) {
+		boError(260) << k_funcinfo << "missing attribute: DataHandlerId for Unit " << i << endl;
+		continue;
+	}
+	bool ok = false;
+	unsigned long int type;
+	unsigned long int id;
+	int dataHandlerId;
+	type = e.attribute(QString::fromLatin1("UnitType")).toULong(&ok);
+	if (!ok) {
+		boError(260) << k_funcinfo << "Invalid UnitType number for Unit " << i << endl;
+		continue;
+	}
+	id = e.attribute(QString::fromLatin1("Id")).toULong(&ok);
+	if (!ok) {
+		boError(260) << k_funcinfo << "Invalid Id number for Unit " << i << endl;
+		continue;
+	}
+	dataHandlerId = e.attribute(QString::fromLatin1("DataHandlerId")).toInt(&ok);
+	if (!ok) {
+		boError(260) << k_funcinfo << "Invalid DataHandlerId number for Unit " << i << endl;
+		continue;
+	}
+
+	// Create unit with Boson
+	Unit* unit = ((Boson*)game())->loadUnit(type, this);
+
+	// Set additional properties
+	d->mUnits.append(unit);
+	unit->dataHandler()->registerHandler(dataHandlerId, this,
+			SLOT(sendProperty(int, QDataStream&, bool*)),
+			SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
+	unit->setId(id);
+
+	// Call unit's loading methods
+	if (!unit->loadFromXML(e)) {
+		boWarning(260) << k_funcinfo << "Could not load unit " << id << " correctly" << endl;
+	}
+
+	// Increase unit count
+	if (unit->isMobile()) {
+		d->mMobilesCount++;
+	} else {
+		d->mFacilitiesCount++;
+	}
+ }
+ return true;
+}
+
+void Player::saveFogOfWar(QDomElement& root) const
+{
+ // AB: I've tried several ways of doing this but couldn't find the solution. so
+ // now we simply stream every single bit as a complete byte. not good, but
+ // works
+ // also note that i don't want to do the bit magic myself, as i want to avoid
+ // this trouble with strings/xml and e.g. \0
+ QDomDocument doc = root.ownerDocument();
+ QDomElement fog = doc.createElement(QString::fromLatin1("Fogged"));
+ QString text;
+ QTextOStream s(&text);
+ s << (Q_UINT32)d->mFogged.size();
+ s << ' ';
+ for (unsigned int i = 0; i < d->mFogged.size(); i++) {
+	s << (char)(d->mFogged.testBit(i) ? '1' : '0');
+ }
+ QDomCDATASection fow = doc.createCDATASection(text);
+ fog.appendChild(fow);
+ root.appendChild(fog);
+}
+
+void Player::loadFogOfWar(const QDomElement& root)
+{
+ QDomElement element = root.namedItem(QString::fromLatin1("Fogged")).toElement();
+ if (element.isNull()) {
+	boError() << k_funcinfo << "No Fogged tag found" << endl;
+	d->mFogged.fill(true);
+	return;
+ }
+ QString text = element.text();
+ if (text.isEmpty()) {
+	boError() << k_funcinfo << "no content for Fogged tag found" << endl;
+	d->mFogged.fill(true);
+	return;
+ }
+ QTextIStream s(&text);
+ s.flags(s.flags() & ~QTextStream::skipws);
+ Q_UINT32 size;
+ s >> size;
+
+ QChar space; // note: QTextStream skips whitespaces for char, not for QChar
+ s >> space;
+ if (space != ' ') {
+	boError() << k_funcinfo << "oops - format error. read: " << space.latin1() << " should be: " << ' ' << endl;
+	// we can not even resize the flog array, because it may be just any
+	// random value. we can't depend on it being correct
+	return;
+ }
+ d->mFogged.resize(size);
+ for (unsigned int i = 0; i < size; i++) {
+	char bit;
+	s >> bit;
+	d->mFogged.setBit(i, bit == '1');
+ }
+}
+
