@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,10 @@ class UnitPlugin;
 class RepairPlugin;
 class BosonParticleSystem;
 class BosonWeapon;
+class BoVector3;
 template<class T> class QValueList;
+template<class T> class QPtrList;
+class QDomElement;
 
 class KGameUnitDebug;
 
@@ -58,11 +61,14 @@ public:
 		IdMoveDestY = UnitBase::IdLast + 4,
 		IdMoveRange = UnitBase::IdLast + 5,
 		IdWantedRotation = UnitBase::IdLast + 6,
+		IdMoveAttacking = UnitBase::IdLast + 7,
+		IdSearchPath = UnitBase::IdLast + 8,
 
 		// properties in MobileUnit
 		IdSpeed = UnitBase::IdLast + 50,
 		IdMovingFailed = UnitBase::IdLast + 51,
 		IdPathRecalculated = UnitBase::IdLast + 52,
+		IdPathAge = UnitBase::IdLast + 53,
 
 		// properties in Facility
 		IdConstructionStep = UnitBase::IdLast + 100,
@@ -75,18 +81,6 @@ public:
 		IdHarvestingType = UnitBase::IdLast + 304
 	};
 
-	enum AnimationModes {
-		// the numbers are just cosmetic and do not need to be in order.
-		// the idle mode is an exception - it MUST be 0.
-		// AB: TODO: add BosonItem::disableAnimation for the
-		// construction animations! they are done differently
-		AnimationIdle = 0, // default mode
-		AnimationMove = 1,
-		AnimationMine = 2,
-		AnimationRefine = 3,
-		AnimationWreckage = 100
-	};
-
 	Unit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
 	virtual ~Unit();
 
@@ -234,7 +228,7 @@ public:
 	Unit* target() const;
 	virtual void setTarget(Unit* target);
 
-	bool inRange(BosonWeapon* w, Unit* unit) const;
+	bool inRange(long unsigned int, Unit* unit) const;
 
 // waypoint stuff: // also in facility - produced units receive this initial waypoint
 	/**
@@ -275,8 +269,9 @@ public:
 	 * @ref work is cleared. After the path is found the unit starts to get
 	 * animated and to move to the destination.
 	 * @param pos The point on the canvas to move to.
+	 * @param attack If this is true, unit will stop and attack any enemy units in range while moving
 	 **/
-	virtual void moveTo(const QPoint& pos);
+	virtual void moveTo(const QPoint& pos, bool attack = false);
 	
 	/**
 	 * Nearly similar to the above version (actually this is called by the
@@ -290,9 +285,10 @@ public:
 	 * allowed to go to. You must use range > 0 e.g. if destination is
 	 * occupied and you should use e.g. @ref weaponRange if the unit should
 	 * attack. -1 keeps the previously set range.
+	 * @param attack Whether to stop and attack any enemy units in range while moving
 	 * @return true if unit can go to destination, false otherwise
 	 **/
-	bool moveTo(float x, float y, int range = 0);
+	bool moveTo(float x, float y, int range = 0, bool attack = false);
 
 	/**
 	 * Turns unit smoothly to given degrees
@@ -306,24 +302,25 @@ public:
 	virtual void stopMoving();
 	void stopAttacking();
 
+	virtual bool saveAsXML(QDomElement& root);
+	virtual bool loadFromXML(const QDomElement& root);
 	virtual bool save(QDataStream& stream);
 	virtual bool load(QDataStream& stream);
 
 	void updateSelectBox();
 
 	/**
-	 * @return All units except this that are in @ref weaponRange of this
-	 * unit.
+	 * @return All units except this that are @ref range or less cells away from this unit
 	 * Note that it doesn't include units on cells that are not visible
 	 * (e.g. fogged) for local player.
 	 **/
-	BoItemList unitsInRange(BosonWeapon* w) const;
+	BoItemList* unitsInRange(unsigned long int range) const;
 
 	/**
 	 * @return Just like @ref unitsInRange but only enemy units.
 	 * Note that it doesn't include units on fogged cells (see @ref unitsInRange)
 	 **/
-	BoItemList enemyUnitsInRange(BosonWeapon* w) const;
+	BoItemList* enemyUnitsInRange(unsigned long int range) const;
 
 	/**
 	 * Calls @ref BosonCanvas setWorkChanged
@@ -342,13 +339,38 @@ public:
 
 	void playSound(UnitSoundEvent event);
 
-	BosonParticleSystem* smokeParticleSystem() const;
-	void setSmokeParticleSystem(BosonParticleSystem* s);
+	/**
+	 * @return List of active particle systems this unit has.
+	 * This may include e.g. smoke for factories.
+	 **/
+	virtual QPtrList<BosonParticleSystem>* particleSystems() const;
+	void setParticleSystems(QPtrList<BosonParticleSystem> list);
 
 	void loadWeapons();
 
 	bool canShootAt(Unit* u);
 
+	/**
+	 * Attack enemy units in range
+	 * @return whether there was any enemy in range
+	 **/
+	bool attackEnemyUnitsInRange();
+
+	/**
+	 * @return Best enemy unit in range to attack
+	 **/
+	Unit* bestEnemyUnitInRange();
+
+	/**
+	 * @return Square of distance between center points of this unit and u
+	 **/
+	int distance(const Unit* u) const;
+	/**
+	 * @return Square of distance between center point of this unit and pos
+	 **/
+	int distance(const BoVector3& pos) const;
+
+
 protected:
 	void shootAt(BosonWeapon* w, Unit* target);
 
@@ -356,7 +378,7 @@ protected:
 	 * @return a list of interesting collisions, i.e. no non-units, no
 	 * destryed units, ...
 	 **/
-	QValueList<Unit*> unitCollisions(bool exact = false) const;
+	QValueList<Unit*> unitCollisions(bool exact = false);
 
 	/** 
 	 * Finds new path to destination.
@@ -380,10 +402,31 @@ protected:
 	 **/
 	virtual void advanceMoveCheck() { }
 
-	BosonWeapon* activeWeapon() const;
+	/**
+	 * @return Whether unit should attack any enemy units in range while moving
+	 **/
+	int moveAttacking() const;
+
+	/**
+	 * @return Whether new path should be searched
+	 **/
+	int searchPath() const;
+	void setSearchPath(int search);
+
+	/**
+	 * Called when the unit wants to move to another position. It may be
+	 * possible that the heights of the cells at the new position are
+	 * different and therefore the unit needs to move in z direction and
+	 * maybe rotate around x and y axis.
+	 *
+	 * Currently rotateX and rotateY are set to 0.0, since it is not yet
+	 * implemented. moveZ is simply set to the highest z value of the
+	 * corners of all cells it occupies.
+	 **/
+	void updateZ(float moveByX, float moveByY, float* moveZ, float* rotateX, float* rotateY);
+
+	virtual const QColor* teamColor() const;
 
-protected:
-	bool mSearchPath;
 
 private:
 	typedef void (Unit::*MemberFunction)(unsigned int advanceCount);
@@ -447,6 +490,8 @@ public:
 
 	virtual QRect boundingRect() const;
 
+	virtual bool saveAsXML(QDomElement& root);
+	virtual bool loadFromXML(const QDomElement& root);
 	virtual bool load(QDataStream& stream);
 	virtual bool save(QDataStream& stream);
 
@@ -498,6 +543,10 @@ public:
 	 **/
 	bool isConstructionComplete() const;
 
+	/**
+	 * @return A percentage that describes how far the construction progress
+	 * of this facility is.
+	 **/
 	double constructionProgress() const;
 
 	/**
@@ -536,8 +585,10 @@ public:
 	 **/
 	virtual UnitPlugin* plugin(int pluginType) const;
 
-	BosonParticleSystem* flamesParticleSystem() const;
-	void setFlamesParticleSystem(BosonParticleSystem* s);
+	virtual bool saveAsXML(QDomElement& root);
+	virtual bool loadFromXML(const QDomElement& root);
+	virtual bool load(QDataStream& stream);
+	virtual bool save(QDataStream& stream);
 
 private:
 	class FacilityPrivate;
