@@ -104,14 +104,14 @@ unsigned long int ProductionPlugin::completedProductionId() const
  if (id == 0) {
 	return 0;
  }
- if(completedProductionType() == ProduceUnit) {
+ if (completedProductionType() == ProduceUnit) {
 	if (mProductionState < speciesTheme()->unitProperties(id)->productionTime()) {
-		boDebug() << "not yet completed: " << id << endl;
+//		boDebug() << k_funcinfo << "not yet completed: " << id << endl;
 		return 0;
 	}
  } else {
 	if (mProductionState < speciesTheme()->technology(id)->productionTime()) {
-		boDebug() << "not yet completed: " << id << endl;
+//		boDebug() << k_funcinfo << "not yet completed: " << id << endl;
 		return 0;
 	}
  }
@@ -185,48 +185,6 @@ double ProductionPlugin::productionProgress() const
  return percentage;
 }
 
-bool ProductionPlugin::canPlaceProductionAt(const QPoint& pos)
-{
- if (!hasProduction() || completedProductionId() <= 0) {
-	boDebug() << k_lineinfo << "no completed construction" << endl;
-	return false;
- }
-
- if(completedProductionType() != ProduceUnit) {
-	boDebug() << k_funcinfo << "Current production is not unit!" << endl;
-	return false;  // Maybe return true, so that tech. is researched? OTOH, this should never be reached anyway ;-)
- }
-
- QValueList<Unit*> list = canvas()->unitCollisionsInRange(pos, BUILD_RANGE);
-
- const UnitProperties* prop = speciesTheme()->unitProperties(currentProductionId());
- if (!prop) {
-	boError() << k_lineinfo << "NULL unit properties - EVIL BUG!" << endl;
-	return false;
- }
- if (prop->isFacility()) {
-	// a facility can be placed within BUILD_RANGE of *any* friendly
-	// facility on map
-	for (unsigned int i = 0; i < list.count(); i++) {
-		if (list[i]->isFacility() && list[i]->owner() == player()) {
-			boDebug() << "Facility in BUILD_RANGE" << endl;
-			// TODO: also check whether a unit is already at that position!!
-			return true;
-		}
-	}
- } else {
-	// a mobile unit can be placed within BUILD_RANGE of its factory *only*
-	for (unsigned int i = 0; i < list.count(); i++) {
-		if (list[i] == (Unit*)this) {
-			// TODO: also check whether a unit is already at that position!!
-			return true;
-		}
-	}
- }
- return false;
-}
-
-
 void ProductionPlugin::advance(unsigned int)
 {
  unsigned long int id = currentProductionId();
@@ -257,8 +215,8 @@ void ProductionPlugin::advance(unsigned int)
 
 		if(currentProductionType() != ProduceUnit) {
 			// It's technology
-			player()->technologyResearched(this, id);
 			removeProduction();
+			player()->technologyResearched(this, id);
 			return;
 		}
 
@@ -273,10 +231,14 @@ void ProductionPlugin::advance(unsigned int)
 			return;
 		}
 		if (speciesTheme()->unitProperties(id)->isFacility()) {
-			game()->slotAddChatSystemMessage(i18n("A %1 has been produced - place it on the map to start construction!").arg(prop->name()));
+			if (player() == boGame->localPlayer()) {
+				game()->slotAddChatSystemMessage(i18n("A %1 has been produced - place it on the map to start construction!").arg(prop->name()));
+			}
 			return;
 		} else {
-			game()->slotAddChatSystemMessage(i18n("A %1 has been produced and will be placed on the map now").arg(prop->name()));
+			if (player() == boGame->localPlayer()) {
+				game()->slotAddChatSystemMessage(i18n("A %1 has been produced and will be placed on the map now").arg(prop->name()));
+			}
 		}
 		int tilex, tiley; // Position of lower-left corner of facility in tiles
 		int theight, twidth; // height and width of facility in tiles
@@ -289,24 +251,23 @@ void ProductionPlugin::advance(unsigned int)
 		int ctry; // Current try
 		currentx = tilex - 1;
 		currenty = tiley - 1;
-		for(int i=1; i <= BUILD_RANGE; i++) {
+		for (int i = 1; i <= BUILD_RANGE; i++) {
 			tries = 2 * i * twidth + 2 * i * theight + 4;
 			currenty++;
-			for(ctry = 1; ctry <= tries; ctry++) {
-				if(ctry <= twidth + i) {
+			for (ctry = 1; ctry <= tries; ctry++) {
+				if (ctry <= twidth + i) {
 					currentx++;
-				} else if(ctry <= twidth + i + theight + 2 * i - 1) {
+				} else if (ctry <= twidth + i + theight + 2 * i - 1) {
 					currenty--;
-				} else if(ctry <= twidth + i + 2 * (theight + 2 * i - 1)) {
+				} else if (ctry <= twidth + i + 2 * (theight + 2 * i - 1)) {
 					currentx--;
-				} else if(ctry <= twidth + i + 3 * (theight + 2 * i - 1)) {
+				} else if (ctry <= twidth + i + 3 * (theight + 2 * i - 1)) {
 					currenty++;
 				} else {
 					currentx++;
 				}
 
-//				FIXME: should not depend on Facility*
-				if (canvas()->canPlaceUnitAt(speciesTheme()->unitProperties(id), QPoint(currentx, currenty), this)) {
+				if (canvas()->canPlaceUnitAtCell(speciesTheme()->unitProperties(id), QPoint(currentx, currenty), this)) {
 					// Free cell - place unit at it
 					mProductionState = mProductionState + 1;
 					//FIXME: buildProduction should not
@@ -317,7 +278,9 @@ void ProductionPlugin::advance(unsigned int)
 			}
 		}
 		boDebug() << "Cannot find free cell around facility :-(" << endl;
-		game()->slotAddChatSystemMessage(i18n("%1 could not be placed on the map - no free cell found. Place it manuall!").arg(prop->name()));
+		if (player() == boGame->localPlayer()) {
+			game()->slotAddChatSystemMessage(i18n("%1 could not be placed on the map - no free cell found. Place it manuall!").arg(prop->name()));
+		}
 	} else {
 		mProductionState = mProductionState + 1;
 	}
@@ -431,15 +394,18 @@ void HarvesterPlugin::advanceMine()
 	unit()->setWork(Unit::WorkNone);
 	return;
  }
+ // Check if unit is at mining location. If not, go there
  if((mResourcesX / BO_TILE_SIZE != unit()->x() / BO_TILE_SIZE) || (mResourcesY / BO_TILE_SIZE != unit()->y() / BO_TILE_SIZE))
  {
 	unit()->moveTo(mResourcesX, mResourcesY);
 	unit()->setAdvanceWork(Unit::WorkMove);
 	return;
  }
+ // Check if we should harvest more
  if (resourcesMined() < prop->maxResources()) {
 	if (canMine(canvas()->cellAt(unit()))) {
-		const int step = (resourcesMined() + 10 <= prop->maxResources()) ? 10 : prop->maxResources() - resourcesMined();
+		// How much more to harvest
+		const int step = (resourcesMined() + miningSpeed() <= prop->maxResources()) ? miningSpeed() : prop->maxResources() - resourcesMined();
 		mResourcesMined = resourcesMined() + step;
 		if (prop->canMineMinerals()) {
 			player()->statistics()->increaseMinedMinerals(step);
@@ -448,12 +414,14 @@ void HarvesterPlugin::advanceMine()
 		}
 		boDebug() << "resources mined: " << resourcesMined() << endl;
 	} else {
+		// This unit cannot mine at this cell
 		boDebug() << k_funcinfo << "cannot mine here" << endl;
 		unit()->setWork(Unit::WorkNone);
 		unit()->setAdvanceWork(Unit::WorkNone);
 		return;
 	}
  } else {
+	// Back to refinery
 	boDebug() << k_funcinfo << "Maximal amount of resources mined." << endl;
 	mHarvestingType = 2; // refining
  }
@@ -461,6 +429,7 @@ void HarvesterPlugin::advanceMine()
 
 void HarvesterPlugin::advanceRefine()
 {
+ // This is the second step of harvesting: returning to refinery and unloading
  boDebug() << k_funcinfo << endl;
  if (resourcesMined() == 0) {
 	boDebug() << k_funcinfo << "refining done" << endl;
@@ -473,8 +442,8 @@ void HarvesterPlugin::advanceRefine()
 	return;
  }
  if (!refinery()) {
-	QPtrList<Unit> list = player()->allUnits();
-	QPtrListIterator<Unit> it(list);
+	// Refinery is not yet set. Find the closest one.
+	QPtrListIterator<Unit> it(*(player()->allUnits()));
 	const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
 	if (!prop) {
 		boError() << k_funcinfo << "NULL harvester plugin" << endl;
@@ -482,19 +451,24 @@ void HarvesterPlugin::advanceRefine()
 		unit()->setAdvanceWork(Unit::WorkNone);
 		return;
 	}
-	Facility* ref = 0; // FIXME: must not depend on Facility! Use a RefineryPlugin
+	Unit* ref = 0;
 	float refdist = 0;
+	RefineryProperties* rprop;
+	boDebug() << k_funcinfo << "going throught units list. count: " << it.count() << endl;
 	while (it.current()) {
-		const UnitProperties* unitProp = it.current()->unitProperties();
-		if (!it.current()->isFacility()) {
+		boDebug() << k_funcinfo << "        testing unit " << it.current()->id() << endl;
+		rprop = (RefineryProperties*)it.current()->properties(PluginProperties::Refinery);
+		if (!rprop) {
 			++it;
 			continue;
 		}
-		if ((prop->canMineMinerals() && unitProp->canRefineMinerals()) || (prop->canMineOil() && unitProp->canRefineOil())) {
+		boDebug() << k_funcinfo << "    unit " << it.current()->id() << " has refinery properties" << endl;
+		if ((prop->canMineMinerals() && rprop->canRefineMinerals()) || (prop->canMineOil() && rprop->canRefineOil())) {
+			boDebug() << k_funcinfo << "    unit " << it.current()->id() << " would be suitable..." << endl;
 			float dist = QMAX(QABS(unit()->x() - it.current()->x()), QABS(unit()->y() - it.current()->y()));
 			if((dist < refdist) || (refdist == 0)) {
 				refdist = dist;
-				ref = (Facility*)it.current();
+				ref = it.current();
 			}
 		}
 		++it;
@@ -509,8 +483,9 @@ void HarvesterPlugin::advanceRefine()
 	}
 	return;
  } else {
+	// Refinery is set
 	if (unit()->isNextTo(refinery())) {
-		const int step = (resourcesMined() >= 10) ? 10 : resourcesMined();
+		const int step = (resourcesMined() >= unloadingSpeed()) ? unloadingSpeed() : resourcesMined();
 		mResourcesMined = resourcesMined() - step;
 		const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
 		if (!prop) {
@@ -526,6 +501,7 @@ void HarvesterPlugin::advanceRefine()
 			player()->statistics()->increaseRefinedOil(step);
 		}
 	} else {
+		// Move next to refinery
 		if (!unit()->moveTo(refinery()->x(), refinery()->y(), 1)) {
 			boDebug() << k_funcinfo << "Cannot find way to refinery" << endl;
 			unit()->setWork(Unit::WorkNone);
@@ -557,9 +533,13 @@ void HarvesterPlugin::refineAt(Unit* refinery)
 	boError() << k_funcinfo << "NULL refinery" << endl;
 	return;
  }
- if (!refinery->unitProperties()->canRefineMinerals() &&
-		!refinery->unitProperties()->canRefineOil()) {
-	boError() << k_funcinfo << refinery->id() << " not a refinery" << endl;
+ RefineryProperties* prop = (RefineryProperties*)refinery->properties(PluginProperties::Refinery);
+ if (!prop) {
+	boError() << k_funcinfo << refinery->id() << " doesn't have refinery properties" << endl;
+	return;
+ }
+ if (!(canMineMinerals() && prop->canRefineMinerals()) && !(canMineOil() && prop->canRefineOil())) {
+	boError() << k_funcinfo << refinery->id() << " not a suitable refinery" << endl;
  }
  boDebug() << k_funcinfo << endl;
  setRefinery(refinery);
@@ -582,16 +562,7 @@ void HarvesterPlugin::setRefinery(Unit* refinery)
  if (!refinery || refinery->isDestroyed()) {
 	return;
  }
- const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
- if (!prop) {
-	return;
- }
- const UnitProperties* refProp = refinery->unitProperties(); // TODO: replace by pluginProperties
- if (prop->canMineMinerals() && refProp->canRefineMinerals()) {
-	mRefinery = refinery;
- } else if (prop->canMineOil() && refProp->canRefineOil()) {
-	mRefinery = refinery;
- }
+ mRefinery = refinery;
 }
 
 bool HarvesterPlugin::canMine(Cell* cell) const
@@ -599,6 +570,7 @@ bool HarvesterPlugin::canMine(Cell* cell) const
  if (canMineMinerals() && cell->groundType() == Cell::GroundGrassMineral) {
 	return true;
  }
+ // FIXME: there's no "oil cells" anymore
  if (canMineOil() && cell->groundType() == Cell::GroundGrassOil) {
 	return true;
  }
@@ -627,8 +599,25 @@ unsigned int HarvesterPlugin::maxResources() const
 {
  const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
  if (!prop) {
-	return false;
+	return 0;
  }
  return prop->maxResources();
 }
 
+unsigned int HarvesterPlugin::miningSpeed() const
+{
+ const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
+ if (!prop) {
+	return 0;
+ }
+ return prop->miningSpeed();
+}
+
+unsigned int HarvesterPlugin::unloadingSpeed() const
+{
+ const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
+ if (!prop) {
+	return 0;
+ }
+ return prop->unloadingSpeed();
+}
