@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 #include "speciestheme.h"
 #include "pluginproperties.h"
-#include "upgradeproperties.h"
 #include "bosonparticlesystem.h"
 #include "bosonparticlesystemproperties.h"
 #include "bosonweapon.h"
@@ -48,11 +47,11 @@ public:
 	QMap<QString, QString> mTextureNames;
 	QMap<int, QString> mSounds;
 
-	QPtrList<UpgradeProperties> mUpgrades;
-	QPtrList<UpgradeProperties> mNotResearchedUpgrades;
-
 	QPtrList<BosonParticleSystemProperties> mDestroyedParticleSystems;
 	QValueList<unsigned long int> mDestroyedParticleSystemIds;
+	QPtrList<BosonParticleSystemProperties> mConstructedParticleSystems;
+	QValueList<unsigned long int> mConstructedParticleSystemIds;
+	BoVector3 mHitPoint;  // FIXME: better name
 };
 
 class UnitProperties::MobileProperties
@@ -74,8 +73,6 @@ public:
 	{
 	}
 
-	bool mCanRefineMinerals;
-	bool mCanRefineOil;
 	unsigned int mConstructionFrames;
 };
 
@@ -106,9 +103,6 @@ void UnitProperties::init()
  mMobileProperties = 0;
  mFacilityProperties = 0;
  d->mPlugins.setAutoDelete(true);
- d->mUpgrades.setAutoDelete(true);
- d->mNotResearchedUpgrades.setAutoDelete(true);
- d->mDestroyedParticleSystems.setAutoDelete(true);
 }
 
 UnitProperties::~UnitProperties()
@@ -151,10 +145,16 @@ void UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
  mSupportMiniMap = conf.readBoolEntry("SupportMiniMap", false);
  isFacility = conf.readBoolEntry("IsFacility", false);
  d->mRequirements = BosonConfig::readUnsignedLongNumList(&conf, "Requirements");
+ mExplodingDamage = conf.readLongNumEntry("ExplodingDamage", 0);
+ mExplodingDamageRange = (float)(conf.readDoubleNumEntry("ExplodingDamageRange", 0));
+ d->mHitPoint = BoVector3::load(&conf, "HitPoint");  // FIXME: better name
+ d->mHitPoint.cellToCanvas();
 
  d->mDestroyedParticleSystemIds = BosonConfig::readUnsignedLongNumList(&conf, "DestroyedParticles");
+ d->mConstructedParticleSystemIds = BosonConfig::readUnsignedLongNumList(&conf, "ConstructedParticles");
  if (mFullMode) {
 	d->mDestroyedParticleSystems = BosonParticleSystemProperties::loadParticleSystemProperties(d->mDestroyedParticleSystemIds, mTheme);
+	d->mConstructedParticleSystems = BosonParticleSystemProperties::loadParticleSystemProperties(d->mConstructedParticleSystemIds, mTheme);
  }
 
  if (isFacility) {
@@ -168,7 +168,6 @@ void UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
  loadAllPluginProperties(&conf);
  loadTextureNames(&conf);
  loadSoundNames(&conf);
- loadUpgrades(&conf);
  loadWeapons(&conf);
 }
 
@@ -194,9 +193,15 @@ void UnitProperties::saveUnitType(const QString& fileName)
  conf.writeEntry("SupportMiniMap", mSupportMiniMap);
  conf.writeEntry("IsFacility", isFacility());
  BosonConfig::writeUnsignedLongNumList(&conf, "Requirements", d->mRequirements);
+ conf.writeEntry("ExplodingDamage", mExplodingDamage);
+ conf.writeEntry("ExplodingDamageRange", mExplodingDamageRange);
+ BoVector3 tmpHitPoint(d->mHitPoint);
+ tmpHitPoint.canvasToCell();
+ tmpHitPoint.save(&conf, "HitPoint");
  conf.writeEntry("Producer", mProducer);
 
  BosonConfig::writeUnsignedLongNumList(&conf, "DestroyedParticles", d->mDestroyedParticleSystemIds);
+ BosonConfig::writeUnsignedLongNumList(&conf, "ConstructedParticles", d->mConstructedParticleSystemIds);
 
  if (isFacility()) {
 	saveFacilityProperties(&conf);
@@ -207,7 +212,6 @@ void UnitProperties::saveUnitType(const QString& fileName)
  saveAllPluginProperties(&conf);  // This saves weapons too
  saveTextureNames(&conf);
  saveSoundNames(&conf);
-// saveUpgrades(&conf); // TODO
 }
 
 void UnitProperties::loadMobileProperties(KSimpleConfig* conf)
@@ -230,9 +234,6 @@ void UnitProperties::loadFacilityProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Facility");
  createFacilityProperties();
- mFacilityProperties->mCanRefineMinerals = conf->readBoolEntry("CanRefineMinerals",
-		false);
- mFacilityProperties->mCanRefineOil = conf->readBoolEntry("CanRefineOil", false);
  mFacilityProperties->mConstructionFrames = conf->readUnsignedNumEntry("ConstructionSteps", 20);
 }
 
@@ -247,6 +248,10 @@ void UnitProperties::loadAllPluginProperties(KSimpleConfig* conf)
  if (conf->hasGroup(HarvesterProperties::propertyGroup())) {
 	loadPluginProperties(new HarvesterProperties(this), conf);
  }
+ if (conf->hasGroup(RefineryProperties::propertyGroup())) {
+	boDebug() << k_funcinfo << "loading refinery plugin properties" << endl;
+	loadPluginProperties(new RefineryProperties(this), conf);
+ }
 }
 
 void UnitProperties::loadPluginProperties(PluginProperties* prop, KSimpleConfig* conf)
@@ -280,7 +285,6 @@ void UnitProperties::loadSoundNames(KSimpleConfig* conf)
 {
  d->mSounds.clear();
  conf->setGroup("Sounds");
- d->mSounds.insert(SoundShoot, conf->readEntry("Shoot", "shoot"));
  d->mSounds.insert(SoundOrderMove, conf->readEntry("OrderMove", "order_move"));
  d->mSounds.insert(SoundOrderAttack, conf->readEntry("OrderAttack", "order_attack"));
  d->mSounds.insert(SoundOrderSelect, conf->readEntry("OrderSelect", "order_select"));
@@ -295,18 +299,27 @@ void UnitProperties::loadWeapons(KSimpleConfig* conf)
  int num = conf->readNumEntry("Weapons", 0);
  mCanShootAtAirUnits = false;
  mCanShootAtLandUnits = false;
+ mMaxAirWeaponRange = 0;
+ mMaxLandWeaponRange = 0;
  for (int i = 0; i < num; i++) {
 	conf->setGroup(QString("Weapon_%1").arg(i));
-	BosonWeaponProperties* p = new BosonWeaponProperties(this);
+	BosonWeaponProperties* p = new BosonWeaponProperties(this, i + 1);
 	p->loadPlugin(conf, mFullMode);
 	d->mPlugins.append(p);
 	if (p->canShootAtAirUnits()) {
 		mCanShootAtAirUnits = true;
+		if(p->range() > mMaxAirWeaponRange) {
+			mMaxAirWeaponRange = p->range();
+		}
 	}
 	if (p->canShootAtLandUnits()) {
 		mCanShootAtLandUnits = true;
+		if(p->range() > mMaxLandWeaponRange) {
+			mMaxLandWeaponRange = p->range();
+		}
 	}
  }
+ mMaxWeaponRange = QMAX(mMaxAirWeaponRange, mMaxLandWeaponRange);
 }
 
 void UnitProperties::saveMobileProperties(KSimpleConfig* conf)
@@ -320,8 +333,6 @@ void UnitProperties::saveMobileProperties(KSimpleConfig* conf)
 void UnitProperties::saveFacilityProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Facility");
- conf->writeEntry("CanRefineMinerals", mFacilityProperties->mCanRefineMinerals);
- conf->writeEntry("CanRefineOil", mFacilityProperties->mCanRefineOil);
  conf->writeEntry("ConstructionSteps", mFacilityProperties->mConstructionFrames);
 }
 
@@ -359,7 +370,6 @@ void UnitProperties::saveTextureNames(KSimpleConfig* conf)
 void UnitProperties::saveSoundNames(KSimpleConfig* conf)
 {
  conf->setGroup("Sounds");
- conf->writeEntry("Shoot", d->mSounds[SoundShoot]);
  conf->writeEntry("OrderMove", d->mSounds[SoundOrderMove]);
  conf->writeEntry("OrderAttack", d->mSounds[SoundOrderAttack]);
  conf->writeEntry("OrderSelect", d->mSounds[SoundOrderSelect]);
@@ -408,22 +418,6 @@ QMap<int, QString> UnitProperties::sounds() const
  return d->mSounds;
 }
 
-QPtrList<UpgradeProperties> UnitProperties::possibleUpgrades() const
-{
- return d->mUpgrades;
-}
-
-QPtrList<UpgradeProperties> UnitProperties::unresearchedUpgrades() const
-{
- return d->mNotResearchedUpgrades;
-}
-
-void UnitProperties::upgradeResearched(UpgradeProperties* upgrade)
-{
- d->mNotResearchedUpgrades.removeRef(upgrade);
-}
-
-
 void UnitProperties::setRequirements(QValueList<unsigned long int> requirements)
 {
  d->mRequirements = requirements;
@@ -463,22 +457,6 @@ bool UnitProperties::canGoOnWater() const
  return mMobileProperties->mCanGoOnWater;
 }
 
-bool UnitProperties::canRefineMinerals() const
-{
- if (!mFacilityProperties) {
-	return false;
- }
- return mFacilityProperties->mCanRefineMinerals;
-}
-
-bool UnitProperties::canRefineOil() const
-{
- if (!mFacilityProperties) {
-	return false;
- }
- return mFacilityProperties->mCanRefineOil;
-}
-
 unsigned int UnitProperties::constructionSteps() const
 {
  if (!mFacilityProperties) {
@@ -498,33 +476,29 @@ const PluginProperties* UnitProperties::properties(int pluginType) const
  return 0;
 }
 
-void UnitProperties::loadUpgrades(KSimpleConfig* conf)
-{
-// boDebug() << k_funcinfo << endl;
- d->mUpgrades.clear();
- d->mNotResearchedUpgrades.clear();
- conf->setGroup("Boson Unit");
- int count = conf->readNumEntry("Upgrades", 0);
- if (count == 0) {
-	return;
- }
- boDebug() << k_funcinfo << "Loading " << count << " upgrades for unit " << typeId() << endl;
- for (int i = 1; i <= count; i++) {
-	UpgradeProperties* upgrade = new UpgradeProperties(this, i);
-	upgrade->loadPlugin(conf);
-	d->mUpgrades.append(upgrade);
-	d->mNotResearchedUpgrades.append(upgrade);
- }
- boDebug() << k_funcinfo << "DONE" << endl;
-}
-
 QPtrList<BosonParticleSystem> UnitProperties::newDestroyedParticleSystems(float x, float y, float z) const
 {
  QPtrList<BosonParticleSystem> list;
  QPtrListIterator<BosonParticleSystemProperties> it(d->mDestroyedParticleSystems);
- while (it.current())
- {
-	list.append(it.current()->newSystem(BoVector3(x, y, z)));
+ while (it.current()) {
+	BosonParticleSystem* s = it.current()->newSystem(BoVector3(x, y, z));
+	if (s) {
+		list.append(s);
+	}
+	++it;
+ }
+ return list;
+}
+
+QPtrList<BosonParticleSystem> UnitProperties::newConstructedParticleSystems(float x, float y, float z) const
+{
+ QPtrList<BosonParticleSystem> list;
+ QPtrListIterator<BosonParticleSystemProperties> it(d->mConstructedParticleSystems);
+ while (it.current()) {
+	BosonParticleSystem* s = it.current()->newSystem(BoVector3(x, y, z));
+	if (s) {
+		list.append(s);
+	}
 	++it;
  }
  return list;
@@ -573,20 +547,6 @@ void UnitProperties::addPlugin(PluginProperties* prop)
  d->mPlugins.append(prop);
 }
 
-void UnitProperties::setCanRefineMinerals(bool r)
-{
- if (mFacilityProperties) {
-	mFacilityProperties->mCanRefineMinerals = r;
- }
-}
-
-void UnitProperties::setCanRefineOil(bool r)
-{
- if (mFacilityProperties) {
-	mFacilityProperties->mCanRefineOil = r;
- }
-}
-
 void UnitProperties::setConstructionSteps(unsigned int steps)
 {
  if (mFacilityProperties) {
@@ -635,6 +595,16 @@ QValueList<unsigned long int> UnitProperties::destroyedParticleSystemIds() const
  return d->mDestroyedParticleSystemIds;
 }
 
+void UnitProperties::setConstructedParticleSystemIds(QValueList<unsigned long int> ids)
+{
+ d->mConstructedParticleSystemIds = ids;
+}
+
+QValueList<unsigned long int> UnitProperties::constructedParticleSystemIds() const
+{
+ return d->mConstructedParticleSystemIds;
+}
+
 void UnitProperties::reset()
 {
  if (mFullMode) {
@@ -660,7 +630,18 @@ void UnitProperties::reset()
  mSupportMiniMap = false;
  d->mRequirements.clear();
  d->mDestroyedParticleSystemIds.clear();
+ d->mConstructedParticleSystemIds.clear();
+ d->mHitPoint.reset();
  mProducer = 0;
+ mExplodingDamage = 0;
+ mExplodingDamageRange = 0;
+ // Delete old mobile/facility properties
+ if (mMobileProperties) {
+	delete mMobileProperties;
+ }
+ if (mFacilityProperties) {
+	delete mFacilityProperties;
+ }
  // Mobile stuff (because unit is mobile by default)
  createMobileProperties();
  mMobileProperties->mSpeed = 0; // Hmm, this doesn't make any sense IMO
@@ -668,7 +649,6 @@ void UnitProperties::reset()
  mMobileProperties->mCanGoOnWater = false;
  // Sounds
  d->mSounds.clear();
- d->mSounds.insert(SoundShoot, "shoot");
  d->mSounds.insert(SoundOrderMove, "order_move");
  d->mSounds.insert(SoundOrderAttack, "order_attack");
  d->mSounds.insert(SoundOrderSelect, "order_select");
@@ -677,6 +657,29 @@ void UnitProperties::reset()
  d->mSounds.insert(SoundReportUnderAttack, "report_underattack");
  // Clear other lists
  d->mTextureNames.clear();
- d->mUpgrades.clear();
- d->mNotResearchedUpgrades.clear();
 }
+
+void UnitProperties::setHitPoint(const BoVector3& hitpoint)
+{
+ d->mHitPoint = hitpoint;
+}
+
+const BoVector3& UnitProperties::hitPoint() const
+{
+ return d->mHitPoint;
+}
+
+const BosonWeaponProperties* UnitProperties::weaponProperties(unsigned long int id) const
+{
+ QPtrListIterator<PluginProperties> it(d->mPlugins);
+ while (it.current()) {
+	if (it.current()->pluginType() == PluginProperties::Weapon) {
+		if (((BosonWeaponProperties*)it.current())->id() == id) {
+			return (BosonWeaponProperties*)it.current();
+		}
+	}
+	++it;
+ }
+ return 0l;
+}
+
